@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -191,17 +191,6 @@ public:
     virtual void RestoreStateVar(CBotStack* &pile,
                                  bool bMain);
 
-    /**
-     * \brief CompCase This routine is defined only for the subclass CBotCase
-     * this allows to make the call on all instructions CompCase to see if it's
-     * a case to the desired value..
-     * \param pj
-     * \param val
-     * \return
-     */
-    virtual bool CompCase(CBotStack* &pj,
-                          int val);
-
     /**
      * \brief SetToken Set the token corresponding to the instruction.
      * \param p
