@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 #include "CBot/CBotInstr/CBotExpression.h"
 #include "CBot/CBotInstr/CBotExprLitBool.h"
+#include "CBot/CBotInstr/CBotExprLitChar.h"
 #include "CBot/CBotInstr/CBotExprLitNan.h"
 #include "CBot/CBotInstr/CBotExprLitNull.h"
 #include "CBot/CBotInstr/CBotExprLitNum.h"
@@ -31,6 +32,7 @@
 #include "CBot/CBotInstr/CBotNew.h"
 #include "CBot/CBotInstr/CBotPostIncExpr.h"
 #include "CBot/CBotInstr/CBotPreIncExpr.h"
+#include "CBot/CBotInstr/CBotTwoOpExpr.h"
 
 #include "CBot/CBotVar/CBotVar.h"
 
@@ -39,13 +41,15 @@
 namespace CBot
 {
 
-////////////////////////////////////////////////////////////////////////////////
-CBotInstr* CBotParExpr::Compile(CBotToken* &p, CBotCStack* pStack)
+CBotInstr* CBotParExpr::Compile(CBotToken* &p, CBotCStack* pStack, bool bConstExpr)
 {
     CBotCStack* pStk = pStack->TokenStack();
 
     pStk->SetStartError(p->GetStart());
 
+    if (bConstExpr)
+        return CBotParExpr::CompileConstExpr(p, pStack);
+
     // is it an expression in parentheses?
     if (IsOfType(p, ID_OPENPAR))
     {
@@ -68,6 +72,13 @@ CBotInstr* CBotParExpr::Compile(CBotToken* &p, CBotCStack* pStack)
     if (inst != nullptr || !pStk->IsOk())
         return pStack->Return(inst, pStk);
 
+    // is it sizeof operator ?
+    inst = CBot::CompileSizeOf(p, pStk);
+    if (inst != nullptr || !pStk->IsOk())
+    {
+        return pStack->Return(inst, pStk);
+    }
+
     // is it a variable name?
     if (p->GetType() == TokenTypVar)
     {
@@ -83,7 +94,7 @@ CBotInstr* CBotParExpr::Compile(CBotToken* &p, CBotCStack* pStack)
 
 
         CBotToken* pvar = p;
-        // no, it an "ordinaty" variable
+        // no, it's an "ordinaty" variable
         inst =  CBotExprVar::Compile(p, pStk);
 
         CBotToken* pp = p;
@@ -151,7 +162,21 @@ CBotInstr* CBotParExpr::CompileLitExpr(CBotToken* &p, CBotCStack* pStack)
     if (p->GetType() == TokenTypNum ||
         p->GetType() == TokenTypDef )
     {
-        CBotInstr* inst = CBotExprLitNum::Compile(p, pStk);
+        CBotInstr* inst = CBot::CompileExprLitNum(p, pStk);
+        return pStack->Return(inst, pStk);
+    }
+
+    // is it sizeof operator ?
+    inst = CBot::CompileSizeOf(p, pStk);
+    if (inst != nullptr || !pStk->IsOk())
+    {
+        return pStack->Return(inst, pStk);
+    }
+
+    // is this a character?
+    if (p->GetType() == TokenTypChar)
+    {
+        CBotInstr* inst = CBotExprLitChar::Compile(p, pStk);
         return pStack->Return(inst, pStk);
     }
 
@@ -202,4 +227,55 @@ CBotInstr* CBotParExpr::CompileLitExpr(CBotToken* &p, CBotCStack* pStack)
     return pStack->Return(nullptr, pStk);
 }
 
+CBotInstr* CBotParExpr::CompileConstExpr(CBotToken* &p, CBotCStack* pStack)
+{
+    CBotCStack* pStk = pStack->TokenStack();
+
+    // is it an expression in parentheses?
+    if (IsOfType(p, ID_OPENPAR))
+    {
+        CBotInstr* inst = CBotTwoOpExpr::Compile(p, pStk, nullptr, true);
+
+        if (nullptr != inst)
+        {
+            if (IsOfType(p, ID_CLOSEPAR))
+            {
+                return pStack->Return(inst, pStk);
+            }
+            pStk->SetError(CBotErrClosePar, p->GetStart());
+        }
+        delete inst;
+        return pStack->Return(nullptr, pStk);
+    }
+
+    // is this a unary operation?
+    CBotInstr* inst = CBotExprUnaire::Compile(p, pStk, true, true);
+    if (inst != nullptr || !pStk->IsOk())
+        return pStack->Return(inst, pStk);
+
+    // is it a number or DefineNum?
+    if (p->GetType() == TokenTypNum ||
+        p->GetType() == TokenTypDef )
+    {
+        CBotInstr* inst = CBot::CompileExprLitNum(p, pStk);
+        return pStack->Return(inst, pStk);
+    }
+
+    // is this a character?
+    if (p->GetType() == TokenTypChar)
+    {
+        CBotInstr* inst = CBotExprLitChar::Compile(p, pStk);
+        return pStack->Return(inst, pStk);
+    }
+
+    // is it sizeof operator ?
+    inst = CBot::CompileSizeOf(p, pStk);
+    if (inst != nullptr || !pStk->IsOk())
+    {
+        return pStack->Return(inst, pStk);
+    }
+
+    return pStack->Return(nullptr, pStk);
+}
+
 } // namespace CBot
