@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -185,8 +185,13 @@ std::string CBotTypResult::ToString()
     switch (m_type)
     {
         case CBotTypVoid: return "void";
+        case CBotTypByte: return "byte";
+        case CBotTypShort: return "short";
+        case CBotTypChar: return "char";
         case CBotTypInt: return "int";
+        case CBotTypLong: return "long";
         case CBotTypFloat: return "float";
+        case CBotTypDouble: return "double";
         case CBotTypBoolean: return "bool";
         case CBotTypString: return "string";
         case CBotTypArrayPointer: return m_next->ToString() + "[]";
