@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -170,7 +170,7 @@ public:
      * \brief Returns the name of the variable
      * \return The name of the variable, empty string if unknown
      */
-    std::string GetName();
+    const std::string& GetName();
 
     /**
      * \brief SetName Changes the name of the variable
@@ -444,12 +444,23 @@ public:
      */
     //@{
 
+    operator bool();
+    operator signed char();
+    operator short();
+    operator uint32_t();
     operator int();
+    operator long();
     operator float();
+    operator double();
     operator std::string();
     void operator=(const CBotVar& var);
+    void operator=(signed char x);
+    void operator=(short x);
+    void operator=(uint32_t x);
     void operator=(int x);
+    void operator=(long x);
     void operator=(float x);
+    void operator=(double x);
     void operator=(const std::string &x);
 
     /**
@@ -465,6 +476,12 @@ public:
      */
     virtual void Copy(CBotVar* pSrc, bool bName = true);
 
+    virtual void SetValByte(signed char val);
+
+    virtual void SetValShort(short val);
+
+    virtual void SetValChar(uint32_t val);
+
     /**
      * \brief Set value as an integer
      *
@@ -475,30 +492,44 @@ public:
      */
     virtual void SetValInt(int val, const std::string& name = "");
 
+    virtual void SetValLong(long val);
+
     /**
      * \brief Set value as float
      * \param val New value
      */
     virtual void SetValFloat(float val);
 
+    virtual void SetValDouble(double val);
+
     /**
      * \brief Set value as string
      * \param val New value
      */
     virtual void SetValString(const std::string& val);
 
+    virtual signed char GetValByte();
+
+    virtual short GetValShort();
+
+    virtual uint32_t GetValChar();
+
     /**
      * \brief Get value as integer
      * \return Current value
      */
     virtual int GetValInt();
 
+    virtual long GetValLong();
+
     /**
      * \brief Get value as float
      * \return Current value
      */
     virtual float GetValFloat();
 
+    virtual double GetValDouble();
+
     /**
      * \brief Get value as string
      *
@@ -623,34 +654,34 @@ public:
 
     /**
      * \brief Save common variable header (name, type, etc.)
-     * \param pf file pointer
+     * \param ostr Output stream
      * \return false on write error
      */
-    virtual bool Save0State(FILE* pf);
+    virtual bool Save0State(std::ostream &ostr);
 
     /**
      * \brief Save variable data
      *
      * Overriden in child classes
      *
-     * \param pf file pointer
+     * \param ostr Output stream
      * \return false on write error
      */
-    virtual bool Save1State(FILE* pf);
+    virtual bool Save1State(std::ostream &ostr);
 
     /**
      * \brief Restore variable
-     * \param pf file pointer
+     * \param istr Input stream
      * \param[out] pVar Pointer to recieve the variable
      * \return false on read error
      */
-    static bool RestoreState(FILE* pf, CBotVar* &pVar);
+    static bool RestoreState(std::istream &istr, CBotVar* &pVar);
 
     //@}
 
 protected:
     //! The corresponding token, defines the variable name
-    CBotToken* m_token;
+    CBotToken* const m_token;
     //! Type of value.
     CBotTypResult m_type;
     //! Initialization status
