@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -40,9 +40,9 @@ void CBotVarBoolean::Not()
     SetValInt(!GetValInt());
 }
 
-bool CBotVarBoolean::Save1State(FILE* pf)
+bool CBotVarBoolean::Save1State(std::ostream &ostr)
 {
-    return WriteWord(pf, m_val);                            // the value of the variable
+    return WriteByte(ostr, m_val);                          // the value of the variable
 }
 
 } // namespace CBot
