@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -22,9 +22,9 @@
 namespace CBot
 {
 
-bool CBotVarFloat::Save1State(FILE* pf)
+bool CBotVarFloat::Save1State(std::ostream &ostr)
 {
-    return WriteFloat(pf, m_val); // the value of the variable
+    return WriteFloat(ostr, m_val); // the value of the variable
 }
 
 } // namespace CBot
