@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -45,67 +45,44 @@ std::string CBotVarInt::GetValString()
 void CBotVarInt::Neg()
 {
     CBotVarNumber::Neg();
-    m_defnum.empty();
+    m_defnum.clear();
 }
 void CBotVarInt::Inc()
 {
     CBotVarNumber::Inc();
-    m_defnum.empty();
+    m_defnum.clear();
 }
 void CBotVarInt::Dec()
 {
     CBotVarNumber::Dec();
-    m_defnum.empty();
+    m_defnum.clear();
 }
 
-void CBotVarInt::XOr(CBotVar* left, CBotVar* right)
-{
-    SetValInt(left->GetValInt() ^ right->GetValInt());
-}
-void CBotVarInt::And(CBotVar* left, CBotVar* right)
-{
-    SetValInt(left->GetValInt() & right->GetValInt());
-}
-void CBotVarInt::Or(CBotVar* left, CBotVar* right)
-{
-    SetValInt(left->GetValInt() | right->GetValInt());
-}
-
-void CBotVarInt::SL(CBotVar* left, CBotVar* right)
-{
-    SetValInt(left->GetValInt() << right->GetValInt());
-}
-void CBotVarInt::ASR(CBotVar* left, CBotVar* right)
-{
-    SetValInt(left->GetValInt() >> right->GetValInt());
-}
 void CBotVarInt::SR(CBotVar* left, CBotVar* right)
 {
-    int source = left->GetValInt();
-    int shift  = right->GetValInt();
-    if (shift >= 1) source &= 0x7fffffff;
-    SetValInt(source >> shift);
+    SetValInt(static_cast<unsigned>(left->GetValInt()) >> right->GetValInt());
 }
 
 void CBotVarInt::Not()
 {
     m_val = ~m_val;
+    m_defnum.clear();
 }
 
-bool CBotVarInt::Save0State(FILE* pf)
+bool CBotVarInt::Save0State(std::ostream &ostr)
 {
     if (!m_defnum.empty())
     {
-        if(!WriteWord(pf, 200)) return false; // special marker
-        if(!WriteString(pf, m_defnum)) return false;
+        if(!WriteWord(ostr, 200)) return false; // special marker
+        if(!WriteString(ostr, m_defnum)) return false;
     }
 
-    return CBotVar::Save0State(pf);
+    return CBotVar::Save0State(ostr);
 }
 
-bool CBotVarInt::Save1State(FILE* pf)
+bool CBotVarInt::Save1State(std::ostream &ostr)
 {
-    return WriteWord(pf, m_val);
+    return WriteInt(ostr, m_val);
 }
 
 } // namespace CBot
