@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 #include "app/controller.h"
 #include "app/input.h"
+#include "app/modman.h"
 #include "app/pathman.h"
 
 #include "common/config_file.h"
@@ -113,7 +114,8 @@ CApplication::CApplication(CSystemUtils* systemUtils)
       m_private(MakeUnique<ApplicationPrivate>()),
       m_configFile(MakeUnique<CConfigFile>()),
       m_input(MakeUnique<CInput>()),
-      m_pathManager(MakeUnique<CPathManager>(systemUtils))
+      m_pathManager(MakeUnique<CPathManager>(systemUtils)),
+      m_modManager(MakeUnique<CModManager>(this, m_pathManager.get()))
 {
     m_exitCode      = 0;
     m_active        = false;
@@ -220,6 +222,35 @@ CSoundInterface* CApplication::GetSound()
     return m_sound.get();
 }
 
+CModManager* CApplication::GetModManager()
+{
+    return m_modManager.get();
+}
+
+void CApplication::LoadEnvironmentVariables()
+{
+    auto dataDir = m_systemUtils->GetEnvVar("COLOBOT_DATA_DIR");
+    if (!dataDir.empty())
+    {
+        m_pathManager->SetDataPath(dataDir);
+        GetLogger()->Info("Using data dir (based on environment variable): '%s'\n", dataDir.c_str());
+    }
+
+    auto langDir = m_systemUtils->GetEnvVar("COLOBOT_LANG_DIR");
+    if (!langDir.empty())
+    {
+        m_pathManager->SetLangPath(langDir);
+        GetLogger()->Info("Using lang dir (based on environment variable): '%s'\n", langDir.c_str());
+    }
+
+    auto saveDir = m_systemUtils->GetEnvVar("COLOBOT_SAVE_DIR");
+    if (!saveDir.empty())
+    {
+        m_pathManager->SetSavePath(saveDir);
+        GetLogger()->Info("Using save dir (based on environment variable): '%s'\n", saveDir.c_str());
+    }
+}
+
 ParseArgsStatus CApplication::ParseArguments(int argc, char *argv[])
 {
     enum OptionType
@@ -286,15 +317,18 @@ ParseArgsStatus CApplication::ParseArguments(int argc, char *argv[])
                 GetLogger()->Message("\n");
                 GetLogger()->Message("%s\n", COLOBOT_FULLNAME);
                 GetLogger()->Message("\n");
-                GetLogger()->Message("List of available options:\n");
+                GetLogger()->Message("List of available options and environment variables:\n");
                 GetLogger()->Message("  -help               this help\n");
                 GetLogger()->Message("  -debug modes        enable debug modes (more info printed in logs; see code for reference of modes)\n");
                 GetLogger()->Message("  -runscene sceneNNN  run given scene on start\n");
                 GetLogger()->Message("  -scenetest          win every mission right after it's loaded\n");
                 GetLogger()->Message("  -loglevel level     set log level to level (one of: trace, debug, info, warn, error, none)\n");
                 GetLogger()->Message("  -langdir path       set custom language directory path\n");
+                GetLogger()->Message("                      environment variable: COLOBOT_LANG_DIR\n");
                 GetLogger()->Message("  -datadir path       set custom data directory path\n");
+                GetLogger()->Message("                      environment variable: COLOBOT_DATA_DIR\n");
                 GetLogger()->Message("  -savedir path       set custom save directory path (must be writable)\n");
+                GetLogger()->Message("                      environment variable: COLOBOT_SAVE_DIR\n");
                 GetLogger()->Message("  -mod path           load datadir mod from given path\n");
                 GetLogger()->Message("  -resolution WxH     set resolution\n");
                 GetLogger()->Message("  -headless           headless mode - disables graphics, sound and user interaction\n");
@@ -486,6 +520,10 @@ bool CApplication::Create()
         GetLogger()->Warn("Config could not be loaded. Default values will be used!\n");
     }
 
+    m_modManager->FindMods();
+    m_modManager->SaveMods();
+    m_modManager->MountAllMods();
+
     // Create the sound instance.
     #ifdef OPENAL_SOUND
     if (!m_headless)
@@ -511,6 +549,8 @@ bool CApplication::Create()
 
     /* SDL initialization sequence */
 
+    // Creating the m_engine now because it holds the vsync flag
+    m_engine = MakeUnique<Gfx::CEngine>(this, m_systemUtils);
 
     Uint32 initFlags = SDL_INIT_VIDEO | SDL_INIT_TIMER;
 
@@ -655,8 +695,6 @@ bool CApplication::Create()
     }
 
     // Create the 3D engine
-    m_engine = MakeUnique<Gfx::CEngine>(this, m_systemUtils);
-
     m_engine->SetDevice(m_device.get());
 
     if (! m_engine->Create() )
@@ -671,21 +709,7 @@ bool CApplication::Create()
     // Create the robot application.
     m_controller = MakeUnique<CController>();
 
-    CThread musicLoadThread([this]()
-    {
-        GetLogger()->Debug("Cache sounds...\n");
-        SystemTimeStamp* musicLoadStart = m_systemUtils->CreateTimeStamp();
-        m_systemUtils->GetCurrentTimeStamp(musicLoadStart);
-
-        m_sound->CacheAll();
-
-        SystemTimeStamp* musicLoadEnd = m_systemUtils->CreateTimeStamp();
-        m_systemUtils->GetCurrentTimeStamp(musicLoadEnd);
-        float musicLoadTime = m_systemUtils->TimeStampDiff(musicLoadStart, musicLoadEnd, STU_MSEC);
-        GetLogger()->Debug("Sound loading took %.2f ms\n", musicLoadTime);
-    },
-    "Sound loading thread");
-    musicLoadThread.Start();
+    StartLoadingMusic();
 
     if (m_runSceneCategory == LevelCategory::Max)
         m_controller->StartApp();
@@ -699,6 +723,15 @@ bool CApplication::Create()
     return true;
 }
 
+void CApplication::ReloadResources()
+{
+    GetLogger()->Info("Reloading resources\n");
+    m_engine->ReloadAllTextures();
+    StartLoadingMusic();
+    m_controller->GetRobotMain()->UpdateCustomLevelList();
+}
+
+
 bool CApplication::CreateVideoSurface()
 {
     Uint32 videoFlags = SDL_WINDOW_OPENGL;
@@ -815,16 +848,45 @@ bool CApplication::CreateVideoSurface()
     m_private->glcontext = SDL_GL_CreateContext(m_private->window);
 
     int vsync = 0;
-    if (GetConfigFile().GetIntProperty("Experimental", "VSync", vsync))
+    if (GetConfigFile().GetIntProperty("Setup", "VSync", vsync))
     {
-        SDL_GL_SetSwapInterval(vsync);
+        m_engine->SetVSync(vsync);
+        TryToSetVSync();
+        vsync = m_engine->GetVSync();
+        GetConfigFile().SetIntProperty("Setup", "VSync", vsync);
 
-        GetLogger()->Info("Using Vsync: %s\n", (vsync ? "true" : "false"));
+        GetLogger()->Info("Using Vsync: %s\n", (vsync == -1 ? "adaptive" : (vsync ? "true" : "false")));
     }
 
     return true;
 }
 
+void CApplication::TryToSetVSync()
+{
+    int vsync = m_engine->GetVSync();
+    int result = SDL_GL_SetSwapInterval(vsync);
+    if (result == -1)
+    {
+        switch (vsync)
+        {
+        case -1:
+            GetLogger()->Warn("Adaptive sync not supported: %s\n", SDL_GetError());
+            m_engine->SetVSync(1);
+            TryToSetVSync();
+            break;
+        case 1:
+            GetLogger()->Warn("Couldn't enable VSync: %s\n", SDL_GetError());
+            m_engine->SetVSync(0);
+            TryToSetVSync();
+            break;
+        case 0:
+            GetLogger()->Warn("Couldn't disable VSync: %s\n", SDL_GetError());
+            m_engine->SetVSync(SDL_GL_GetSwapInterval());
+            break;
+        }
+    }
+}
+
 bool CApplication::ChangeVideoConfig(const Gfx::DeviceConfig &newConfig)
 {
     m_deviceConfig = newConfig;
@@ -833,6 +895,8 @@ bool CApplication::ChangeVideoConfig(const Gfx::DeviceConfig &newConfig)
     SDL_SetWindowSize(m_private->window, m_deviceConfig.size.x, m_deviceConfig.size.y);
     SDL_SetWindowFullscreen(m_private->window, m_deviceConfig.fullScreen ? SDL_WINDOW_FULLSCREEN : 0);
 
+    TryToSetVSync();
+
     m_device->ConfigChanged(m_deviceConfig);
 
     m_eventQueue->AddEvent(Event(EVENT_RESOLUTION_CHANGED));
@@ -994,6 +1058,10 @@ int CApplication::Run()
 
     MoveMouse(Math::Point(0.5f, 0.5f)); // center mouse on start
 
+    SystemTimeStamp *previousTimeStamp = m_systemUtils->CreateTimeStamp();
+    SystemTimeStamp *currentTimeStamp = m_systemUtils->CreateTimeStamp();
+    SystemTimeStamp *interpolatedTimeStamp = m_systemUtils->CreateTimeStamp();
+
     while (true)
     {
         if (m_active)
@@ -1092,21 +1160,30 @@ int CApplication::Run()
 
             CProfiler::StartPerformanceCounter(PCNT_UPDATE_ALL);
 
-            // Prepare and process step simulation event
-            Event event = CreateUpdateEvent();
-            if (event.type != EVENT_NULL && m_controller != nullptr)
+            // Prepare and process step simulation event(s)
+            // If game speed is increased then we do extra ticks per loop iteration to improve physics accuracy.
+            int numTickSlices = static_cast<int>(GetSimulationSpeed());
+            if(numTickSlices < 1) numTickSlices = 1;
+            m_systemUtils->CopyTimeStamp(previousTimeStamp, m_curTimeStamp);
+            m_systemUtils->GetCurrentTimeStamp(currentTimeStamp);
+            for(int tickSlice = 0; tickSlice < numTickSlices; tickSlice++)
             {
-                LogEvent(event);
+                m_systemUtils->InterpolateTimeStamp(interpolatedTimeStamp, previousTimeStamp, currentTimeStamp, (tickSlice+1)/static_cast<float>(numTickSlices));
+                Event event = CreateUpdateEvent(interpolatedTimeStamp);
+                if (event.type != EVENT_NULL && m_controller != nullptr)
+                {
+                    LogEvent(event);
 
-                m_sound->FrameMove(m_relTime);
+                    m_sound->FrameMove(m_relTime);
 
-                CProfiler::StartPerformanceCounter(PCNT_UPDATE_GAME);
-                m_controller->ProcessEvent(event);
-                CProfiler::StopPerformanceCounter(PCNT_UPDATE_GAME);
+                    CProfiler::StartPerformanceCounter(PCNT_UPDATE_GAME);
+                    m_controller->ProcessEvent(event);
+                    CProfiler::StopPerformanceCounter(PCNT_UPDATE_GAME);
 
-                CProfiler::StartPerformanceCounter(PCNT_UPDATE_ENGINE);
-                m_engine->FrameUpdate();
-                CProfiler::StopPerformanceCounter(PCNT_UPDATE_ENGINE);
+                    CProfiler::StartPerformanceCounter(PCNT_UPDATE_ENGINE);
+                    m_engine->FrameUpdate();
+                    CProfiler::StopPerformanceCounter(PCNT_UPDATE_ENGINE);
+                }
             }
 
             CProfiler::StopPerformanceCounter(PCNT_UPDATE_ALL);
@@ -1122,6 +1199,10 @@ int CApplication::Run()
     }
 
 end:
+    m_systemUtils->DestroyTimeStamp(previousTimeStamp);
+    m_systemUtils->DestroyTimeStamp(currentTimeStamp);
+    m_systemUtils->DestroyTimeStamp(interpolatedTimeStamp);
+
     return m_exitCode;
 }
 
@@ -1454,6 +1535,26 @@ void CApplication::InternalResumeSimulation()
     m_absTimeBase = m_exactAbsTime;
 }
 
+void CApplication::StartLoadingMusic()
+{
+    CThread musicLoadThread([this]()
+    {
+        GetLogger()->Debug("Cache sounds...\n");
+        SystemTimeStamp* musicLoadStart = m_systemUtils->CreateTimeStamp();
+        m_systemUtils->GetCurrentTimeStamp(musicLoadStart);
+
+        m_sound->Reset();
+        m_sound->CacheAll();
+
+        SystemTimeStamp* musicLoadEnd = m_systemUtils->CreateTimeStamp();
+        m_systemUtils->GetCurrentTimeStamp(musicLoadEnd);
+        float musicLoadTime = m_systemUtils->TimeStampDiff(musicLoadStart, musicLoadEnd, STU_MSEC);
+        GetLogger()->Debug("Sound loading took %.2f ms\n", musicLoadTime);
+    },
+    "Sound loading thread");
+    musicLoadThread.Start();
+}
+
 bool CApplication::GetSimulationSuspended() const
 {
     return m_simulationSuspended;
@@ -1463,20 +1564,20 @@ void CApplication::SetSimulationSpeed(float speed)
 {
     m_simulationSpeed = speed;
 
-    m_systemUtils->GetCurrentTimeStamp(m_baseTimeStamp);
+    m_systemUtils->CopyTimeStamp(m_baseTimeStamp, m_curTimeStamp);
     m_realAbsTimeBase = m_realAbsTime;
     m_absTimeBase = m_exactAbsTime;
 
     GetLogger()->Info("Simulation speed = %.2f\n", speed);
 }
 
-Event CApplication::CreateUpdateEvent()
+Event CApplication::CreateUpdateEvent(SystemTimeStamp *newTimeStamp)
 {
     if (m_simulationSuspended)
         return Event(EVENT_NULL);
 
     m_systemUtils->CopyTimeStamp(m_lastTimeStamp, m_curTimeStamp);
-    m_systemUtils->GetCurrentTimeStamp(m_curTimeStamp);
+    m_systemUtils->CopyTimeStamp(m_curTimeStamp, newTimeStamp);
 
     long long absDiff = m_systemUtils->TimeStampExactDiff(m_baseTimeStamp, m_curTimeStamp);
     long long newRealAbsTime = m_realAbsTimeBase + absDiff;
@@ -1699,6 +1800,10 @@ char CApplication::GetLanguageChar() const
             langChar = 'E';
             break;
 
+        case LANGUAGE_CZECH:
+            langChar = 'C';
+            break;
+
         case LANGUAGE_GERMAN:
             langChar = 'D';
             break;
@@ -1714,6 +1819,11 @@ char CApplication::GetLanguageChar() const
         case LANGUAGE_RUSSIAN:
             langChar = 'R';
             break;
+
+        case LANGUAGE_PORTUGUESE_BRAZILIAN:
+            langChar = 'B';
+            break;
+
     }
     return langChar;
 }
@@ -1750,6 +1860,10 @@ void CApplication::SetLanguage(Language language)
             {
                 m_language = LANGUAGE_ENGLISH;
             }
+            else if (strncmp(envLang,"cs",2) == 0)
+            {
+                m_language = LANGUAGE_CZECH;
+            }
             else if (strncmp(envLang,"de",2) == 0)
             {
                 m_language = LANGUAGE_GERMAN;
@@ -1766,6 +1880,10 @@ void CApplication::SetLanguage(Language language)
             {
                 m_language = LANGUAGE_RUSSIAN;
             }
+            else if (strncmp(envLang,"pt",2) == 0)
+            {
+                m_language = LANGUAGE_PORTUGUESE_BRAZILIAN;
+            }
             else
             {
                 GetLogger()->Warn("Enviromnent locale ('%s') is not supported, setting default language\n", envLang);
@@ -1781,6 +1899,10 @@ void CApplication::SetLanguage(Language language)
             locale = "";
             break;
 
+        case LANGUAGE_CZECH:
+            locale = "cs_CZ.utf8";
+            break;
+
         case LANGUAGE_ENGLISH:
             locale = "en_US.utf8";
             break;
@@ -1800,6 +1922,10 @@ void CApplication::SetLanguage(Language language)
         case LANGUAGE_RUSSIAN:
             locale = "ru_RU.utf8";
             break;
+
+        case LANGUAGE_PORTUGUESE_BRAZILIAN:
+            locale = "pt_BR.utf8";
+            break;
     }
 
     std::string langStr = "LANGUAGE=";
@@ -1820,7 +1946,12 @@ void CApplication::SetLanguage(Language language)
     // Update C++ locale
     try
     {
+#if defined(_MSC_VER) && defined(_DEBUG)
+        // Avoids failed assertion in VS debugger
+        throw -1;
+#else
         std::locale::global(std::locale(systemLocale.c_str()));
+#endif
     }
     catch (...)
     {
