@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -42,6 +42,7 @@ class CEventQueue;
 class CController;
 class CSoundInterface;
 class CInput;
+class CModManager;
 class CPathManager;
 class CConfigFile;
 class CSystemUtils;
@@ -162,12 +163,18 @@ public:
     CEventQueue* GetEventQueue();
     //! Returns the sound subsystem
     CSoundInterface* GetSound();
+    //! Returns the mod manager
+    CModManager* GetModManager();
 
 public:
-    //! Parses commandline arguments
+    //! Loads some data from environment variables
+    void LoadEnvironmentVariables();
+    //! Parses commandline arguments (they take priority)
     ParseArgsStatus ParseArguments(int argc, char *argv[]);
     //! Initializes the application
     bool        Create();
+    //! Reloads the application resources, e.g. mods
+    void        ReloadResources();
     //! Main event loop
     int         Run();
     //! Returns the code to be returned at main() exit
@@ -281,13 +288,16 @@ public:
 protected:
     //! Creates the window's SDL_Surface
     bool CreateVideoSurface();
+    //! Tries to set the SDL vsync state desired by the 3D engine
+    //! The final state of SDL vsync is set in the 3D engine afterwards
+    void TryToSetVSync();
 
     //! Processes the captured SDL event to Event struct
     Event       ProcessSystemEvent();
     //! If applicable, creates a virtual event to match the changed state as of new event
     Event       CreateVirtualEvent(const Event& sourceEvent);
     //! Prepares a simulation update event
-    TEST_VIRTUAL Event CreateUpdateEvent();
+    TEST_VIRTUAL Event CreateUpdateEvent(SystemTimeStamp *newTimeStamp);
     //! Logs debug data for event
     void        LogEvent(const Event& event);
 
@@ -299,6 +309,9 @@ protected:
     //! Internal procedure to reset time counters
     void InternalResumeSimulation();
 
+    //! Loads music in a new thread
+    void StartLoadingMusic();
+
 protected:
     //! System utils instance
     CSystemUtils* m_systemUtils;
@@ -320,6 +333,8 @@ protected:
     std::unique_ptr<CInput> m_input;
     //! Path manager
     std::unique_ptr<CPathManager> m_pathManager;
+    //! Mod manager
+    std::unique_ptr<CModManager> m_modManager;
 
     //! Code to return at exit
     int             m_exitCode;
