@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -38,48 +38,32 @@
 #include <boost/filesystem.hpp>
 
 CPathManager::CPathManager(CSystemUtils* systemUtils)
-    : m_systemUtils(systemUtils)
+    : m_dataPath(systemUtils->GetDataPath())
+    , m_langPath(systemUtils->GetLangPath())
+    , m_savePath(systemUtils->GetSaveDir())
+    , m_modSearchDirs{}
 {
-    #ifdef PORTABLE
-        m_dataPath = "./data";
-        m_langPath = "./lang";
-        m_savePath = "./saves";
-    #else
-        m_dataPath = m_systemUtils->GetDataPath();
-        m_langPath = m_systemUtils->GetLangPath();
-        #ifdef DEV_BUILD
-            m_savePath = "./saves";
-        #else
-            m_savePath = m_systemUtils->GetSaveDir();
-        #endif
-    #endif
 }
 
 CPathManager::~CPathManager()
 {
 }
 
-void CPathManager::SetDataPath(std::string dataPath)
+void CPathManager::SetDataPath(const std::string &dataPath)
 {
     m_dataPath = dataPath;
 }
 
-void CPathManager::SetLangPath(std::string langPath)
+void CPathManager::SetLangPath(const std::string &langPath)
 {
     m_langPath = langPath;
 }
 
-void CPathManager::SetSavePath(std::string savePath)
+void CPathManager::SetSavePath(const std::string &savePath)
 {
     m_savePath = savePath;
 }
 
-void CPathManager::AddMod(std::string modPath)
-{
-    GetLogger()->Info("Loading mod: '%s'\n", modPath.c_str());
-    CResourceManager::AddLocation(modPath, true);
-}
-
 const std::string& CPathManager::GetDataPath()
 {
     return m_dataPath;
@@ -106,8 +90,8 @@ std::string CPathManager::VerifyPaths()
     {
         GetLogger()->Error("Data directory '%s' doesn't exist or is not a directory\n", m_dataPath.c_str());
         return std::string("Could not read from data directory:\n") +
-        std::string("'") + m_dataPath + std::string("'\n") +
-        std::string("Please check your installation, or supply a valid data directory by -datadir option.");
+            std::string("'") + m_dataPath + std::string("'\n") +
+            std::string("Please check your installation, or supply a valid data directory by -datadir option.");
     }
 
     #if PLATFORM_WINDOWS
@@ -133,19 +117,71 @@ std::string CPathManager::VerifyPaths()
 
 void CPathManager::InitPaths()
 {
-    LoadModsFromDir(m_dataPath+"/mods");
-    LoadModsFromDir(m_savePath+"/mods");
-
     GetLogger()->Info("Data path: %s\n", m_dataPath.c_str());
     GetLogger()->Info("Save path: %s\n", m_savePath.c_str());
-    CResourceManager::AddLocation(m_dataPath, false);
+
+    m_modSearchDirs.push_back(m_dataPath + "/mods");
+    m_modSearchDirs.push_back(m_savePath + "/mods");
+
+    if (!m_modSearchDirs.empty())
+    {
+        GetLogger()->Info("Mod search dirs:\n");
+        for(const std::string& modSearchDir : m_modSearchDirs)
+            GetLogger()->Info("  * %s\n", modSearchDir.c_str());
+    }
+
+    CResourceManager::AddLocation(m_dataPath);
+
     CResourceManager::SetSaveLocation(m_savePath);
-    CResourceManager::AddLocation(m_savePath, true);
+    CResourceManager::AddLocation(m_savePath);
+
+    GetLogger()->Debug("Finished initalizing data paths\n");
+    GetLogger()->Debug("PHYSFS search path is:\n");
+    for (const std::string& path : CResourceManager::GetLocations())
+        GetLogger()->Debug("  * %s\n", path.c_str());
 }
 
-void CPathManager::LoadModsFromDir(const std::string &dir)
+void CPathManager::AddMod(const std::string &path)
 {
-    GetLogger()->Trace("Looking for mods in '%s' ...\n", dir.c_str());
+    m_mods.push_back(path);
+}
+
+std::vector<std::string> CPathManager::FindMods() const
+{
+    std::vector<std::string> mods;
+    GetLogger()->Info("Found mods:\n");
+    for (const auto &searchPath : m_modSearchDirs)
+    {
+        for (const auto &modPath : FindModsInDir(searchPath))
+        {
+            GetLogger()->Info("  * %s\n", modPath.c_str());
+            mods.push_back(modPath);
+        }
+    }
+    GetLogger()->Info("Additional mod paths:\n");
+    for (const auto& modPath : m_mods)
+    {
+        if (boost::filesystem::exists(modPath))
+        {
+            GetLogger()->Info("  * %s\n", modPath.c_str());
+            mods.push_back(modPath);
+        }
+        else
+        {
+            GetLogger()->Warn("Mod does not exist: %s\n", modPath.c_str());
+        }
+    }
+    return mods;
+}
+
+void CPathManager::AddModSearchDir(const std::string &modSearchDirPath)
+{
+    m_modSearchDirs.push_back(modSearchDirPath);
+}
+
+std::vector<std::string> CPathManager::FindModsInDir(const std::string &dir) const
+{
+    std::vector<std::string> ret;
     try
     {
         #if PLATFORM_WINDOWS
@@ -156,9 +192,9 @@ void CPathManager::LoadModsFromDir(const std::string &dir)
         for(; iterator != boost::filesystem::directory_iterator(); ++iterator)
         {
             #if PLATFORM_WINDOWS
-            AddMod(CSystemUtilsWindows::UTF8_Encode(iterator->path().wstring()));
+            ret.push_back(CSystemUtilsWindows::UTF8_Encode(iterator->path().wstring()));
             #else
-            AddMod(iterator->path().string());
+            ret.push_back(iterator->path().string());
             #endif
         }
     }
@@ -166,4 +202,5 @@ void CPathManager::LoadModsFromDir(const std::string &dir)
     {
         GetLogger()->Warn("Unable to load mods from directory '%s': %s\n", dir.c_str(), e.what());
     }
+    return ret;
 }
