@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -202,8 +202,13 @@ enum EventType
     EVENT_DIALOG_LABEL3     = 305,
     EVENT_DIALOG_LIST       = 306,
     EVENT_DIALOG_EDIT       = 307,
-    EVENT_DIALOG_CHECK1     = 308,
-    EVENT_DIALOG_CHECK2     = 309,
+    EVENT_DIALOG_EDIT2      = 308,
+    EVENT_DIALOG_CHECK1     = 309,
+    EVENT_DIALOG_CHECK2     = 310,
+    EVENT_DIALOG_GROUP1     = 320,
+    EVENT_DIALOG_NEWDIR     = 330,
+    EVENT_DIALOG_ACTION     = 348,
+    EVENT_DIALOG_STOP       = 349,
 
     EVENT_INTERFACE_TRAINER = 400,
     EVENT_INTERFACE_DEFI    = 401,
@@ -220,6 +225,8 @@ enum EventType
     EVENT_INTERFACE_ABORT   = 412,
     EVENT_INTERFACE_USER    = 413,
     EVENT_INTERFACE_SATCOM  = 414,
+    EVENT_INTERFACE_PLUS    = 415,
+    EVENT_INTERFACE_MODS    = 416,
 
     EVENT_INTERFACE_CHAP    = 420,
     EVENT_INTERFACE_LIST    = 421,
@@ -252,6 +259,7 @@ enum EventType
     EVENT_INTERFACE_INVERTY = 469,
     EVENT_INTERFACE_EFFECT  = 470,
     EVENT_INTERFACE_BGPAUSE = 471,
+    EVENT_INTERFACE_BGMUTE  = 472,
     EVENT_INTERFACE_FOG     = 474,
     EVENT_INTERFACE_EDITMODE= 476,
     EVENT_INTERFACE_EDITVALUE= 477,
@@ -268,6 +276,7 @@ enum EventType
     EVENT_INTERFACE_SHADOW_MAPPING_QUALITY = 788,
     EVENT_INTERFACE_SHADOW_MAPPING_BUFFER = 789,
     EVENT_INTERFACE_LANGUAGE = 790,
+    EVENT_INTERFACE_VSYNC = 791,
 
     EVENT_INTERFACE_KINFO1  = 500,
     EVENT_INTERFACE_KINFO2  = 501,
@@ -304,6 +313,21 @@ enum EventType
     EVENT_INTERFACE_JOYSTICK_CAM_Y_INVERT = 573,
     EVENT_INTERFACE_JOYSTICK_CAM_Z_INVERT = 574,
 
+    EVENT_INTERFACE_PLUS_TRAINER  = 575,
+    EVENT_INTERFACE_PLUS_RESEARCH = 576,
+    EVENT_INTERFACE_PLUS_EXPLORER = 577,
+
+    EVENT_INTERFACE_MOD_LIST = 580,
+    EVENT_INTERFACE_WORKSHOP = 581,
+    EVENT_INTERFACE_MODS_DIR = 582,
+    EVENT_INTERFACE_MOD_ENABLE_OR_DISABLE = 583,
+    EVENT_INTERFACE_MODS_APPLY = 584,
+    EVENT_INTERFACE_MOD_SUMMARY = 585,
+    EVENT_INTERFACE_MOD_DETAILS = 586,
+    EVENT_INTERFACE_MOD_MOVE_UP = 587,
+    EVENT_INTERFACE_MOD_MOVE_DOWN = 888,
+    EVENT_INTERFACE_MODS_REFRESH = 589,
+
     EVENT_INTERFACE_GLINTl  = 590,
     EVENT_INTERFACE_GLINTr  = 591,
     EVENT_INTERFACE_GLINTu  = 592,
@@ -462,7 +486,7 @@ enum EventType
     EVENT_OBJECT_BNUCLEAR   = 1060,
     EVENT_OBJECT_BPARA      = 1061,
     EVENT_OBJECT_BINFO      = 1062,
-    EVENT_OBJECT_BDESTROYER = 1063,
+    EVENT_OBJECT_BSAFE      = 1063,
     EVENT_OBJECT_GFLAT      = 1070,
     EVENT_OBJECT_FCREATE    = 1071,
     EVENT_OBJECT_FDELETE    = 1072,
@@ -492,6 +516,11 @@ enum EventType
     EVENT_OBJECT_FACTORYrr  = 1098,
     EVENT_OBJECT_FACTORYrs  = 1099,
     EVENT_OBJECT_FACTORYsa  = 1100,
+    EVENT_OBJECT_FACTORYwb  = 1101,
+    EVENT_OBJECT_FACTORYtb  = 1102,
+    EVENT_OBJECT_FACTORYfb  = 1103,
+    EVENT_OBJECT_FACTORYib  = 1104,
+    EVENT_OBJECT_FACTORYtg  = 1105,
     EVENT_OBJECT_SEARCH     = 1200,
     EVENT_OBJECT_TERRAFORM  = 1201,
     EVENT_OBJECT_FIRE       = 1202,
@@ -594,6 +623,21 @@ enum EventType
     EVENT_CODE_BATTLE_START = 2200, //!< button that starts the code battle
     EVENT_CODE_BATTLE_SPECTATOR = 2201, //!< button that controls the code battle spectator camera
 
+    EVENT_OBJECT_RBUILDER       = 2300,
+    EVENT_OBJECT_BUILD          = 2301,
+    EVENT_OBJECT_RTARGET        = 2302,
+
+    //! Buttons that switch viewpoints
+    EVENT_VIEWPOINT0 = 3000,
+    EVENT_VIEWPOINT1 = 3001,
+    EVENT_VIEWPOINT2 = 3002,
+    EVENT_VIEWPOINT3 = 3003,
+    EVENT_VIEWPOINT4 = 3004,
+    EVENT_VIEWPOINT5 = 3005,
+    EVENT_VIEWPOINT6 = 3006,
+    EVENT_VIEWPOINT7 = 3007,
+    EVENT_VIEWPOINT8 = 3008,
+    EVENT_VIEWPOINT9 = 3009,
     //! Maximum value of standard events
     EVENT_STD_MAX,
 
