@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -39,57 +39,11 @@
 namespace Gfx
 {
 
-
-/**
-* \struct VertexAttribute
-* \brief Vertex attribute
-*
-* This structure contains parameters for a vertex attribute.
-*/
-struct VertexAttribute
+enum VertexType
 {
-    //! true enables vertex attribute
-    bool enabled = false;
-    //! true means normalized value (integer types only)
-    bool normalized = false;
-    //! Number of elements in the vertex attribute.
-    //! Valid values are 1, 2, 3, and 4. Depends on specific attribute.
-    unsigned char size = 0;
-    //! Type of values in vertex attribute
-    Type type = Type::UBYTE;
-    //! Offset to the vertex attribute
-    int offset = 0;
-    //! Stride of vertex attribute
-    int stride = 0;
-    //! Default values used when attribute is disabled
-    float values[4] = {0.0f, 0.0f, 0.0f, 0.0f};
-};
-
-/**
-* \struct VertexFormat
-* \brief Vertex format
-*
-* This structure defines vertex formats for generic vertex arrays.
-*
-* It contains:
-*  - vertex coordinate specification
-*  - color specification
-*  - normal specification
-*  - texture coordinate 1 specification
-*  - texture coordinate 2 specification
-*/
-struct VertexFormat
-{
-    //! Vertex coordinate
-    VertexAttribute vertex{};
-    //! Color
-    VertexAttribute color{};
-    //! Normal
-    VertexAttribute normal{};
-    //! Texture coordinate 1
-    VertexAttribute tex1{};
-    //! Texture coordinate 2
-    VertexAttribute tex2{};
+    VERTEX_TYPE_NORMAL,
+    VERTEX_TYPE_TEX2,
+    VERTEX_TYPE_COL,
 };
 
 /**
@@ -105,6 +59,8 @@ struct VertexFormat
  */
 struct Vertex
 {
+    static constexpr VertexType VERTEX_TYPE = VERTEX_TYPE_NORMAL;
+
     Math::Vector coord;
     Math::Vector normal;
     Math::Point texCoord;
@@ -137,6 +93,8 @@ struct Vertex
  */
 struct VertexCol
 {
+    static constexpr VertexType VERTEX_TYPE = VERTEX_TYPE_COL;
+
     Math::Vector coord;
     Color color;
 
@@ -166,6 +124,8 @@ struct VertexCol
  */
 struct VertexTex2
 {
+    static constexpr VertexType VERTEX_TYPE = VERTEX_TYPE_TEX2;
+
     Math::Vector coord;
     Math::Vector normal;
     Math::Point texCoord;
