@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -246,8 +246,8 @@ struct EngineBaseObject
     Math::Vector           bboxMin;
     //! bounding box max (origin 0,0,0 always included)
     Math::Vector           bboxMax;
-    //! Radius of the sphere at the origin
-    float                  radius = 0.0f;
+    //! A bounding sphere that contains all the vertices in this EngineBaseObject
+    Math::Sphere           boundingSphere;
     //! Next tier (Tex)
     std::vector<EngineBaseObjTexTier> next;
 
@@ -1072,6 +1072,13 @@ public:
     bool            GetTerrainShadows();
     //@}
 
+    //@{
+    //! Management of vertical synchronization
+    // NOTE: This is an user configuration setting
+    void            SetVSync(int value);
+    int             GetVSync();
+    //@}
+
     //@{
     //! Management of shadow color
     // NOTE: This is a setting configurable only in INI file
@@ -1161,8 +1168,8 @@ public:
     //! Updates the scene after a change of parameters
     void            ApplyChange();
 
-    void            ClearDisplayCrashSpheres();
-    void            AddDisplayCrashSpheres(const std::vector<Math::Sphere>& crashSpheres);
+    void            RenderDebugSphere(const Math::Sphere&, const Math::Matrix& transform = Math::Matrix{}, const Color& = Color{0.0f, 0.0f, 1.0f, 1.0f});
+    void            RenderDebugBox(const Math::Vector& mins, const Math::Vector& maxs, const Math::Matrix& transform = Math::Matrix{}, const Color& = Color{0.0f, 0.0f, 1.0f, 1.0f});
 
     void            SetDebugLights(bool debugLights);
     bool            GetDebugLights();
@@ -1182,6 +1189,10 @@ public:
     void            EnablePauseBlur();
     void            DisablePauseBlur();
 
+    //! Reloads all textures
+    /** This additionally sends EVENT_RELOAD_TEXTURES to reload all textures not maintained by CEngine **/
+    void ReloadAllTextures();
+
 protected:
     //! Resets some states and flushes textures after device was changed (e.g. resoulution changed)
     /** Instead of calling this directly, send EVENT_RESOLUTION_CHANGED event **/
@@ -1228,7 +1239,7 @@ protected:
     void        DrawStats();
     //! Draw mission timer
     void        DrawTimer();
-    void        DrawCrashSpheres();
+    void        RenderPendingDebugDraws();
 
     //! Creates a new tier 2 object (texture)
     EngineBaseObjTexTier&  AddLevel2(EngineBaseObject& p1, const std::string& tex1Name, const std::string& tex2Name);
@@ -1280,10 +1291,6 @@ protected:
     };
     static void WriteScreenShotThread(std::unique_ptr<WriteScreenShotData> data);
 
-    //! Reloads all textures
-    /** This additionally sends EVENT_RELOAD_TEXTURES to reload all textures not maintained by CEngine **/
-    void ReloadAllTextures();
-
 protected:
     CApplication*     m_app;
     CSystemUtils*     m_systemUtils;
@@ -1336,6 +1343,9 @@ protected:
     //! Texture bias for sampling shadow maps
     Math::Matrix    m_shadowBias;
 
+    //! Vertical synchronization controll
+    int m_vsync;
+
     //! World matrix for 2D interface
     Math::Matrix    m_matWorldInterface;
     //! Projection matrix for 2D interface
@@ -1406,6 +1416,14 @@ protected:
 
     Texture         m_shadowMap;
 
+    struct PendingDebugDraw
+    {
+        std::vector<VertexCol> vertices;
+        std::vector<int> firsts;
+        std::vector<int> counts;
+    }
+    m_pendingDebugDraws;
+
     //! Ranks of highlighted objects
     int             m_highlightRank[100];
     //! Highlight visible?
@@ -1479,7 +1497,6 @@ protected:
 
     std::unordered_map<std::string, int> m_staticMeshBaseObjects;
 
-    std::vector<Math::Sphere> m_displayCrashSpheres;
     std::vector<std::vector<VertexCol>> m_displayGoto;
     std::unique_ptr<CImage> m_displayGotoImage;
 
