@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -77,18 +77,25 @@ enum FontType
     FONT_ITALIC     = 0x08,
 
     //! Default colobot font used for interface
-    FONT_COLOBOT    = 0x00,
+    FONT_COMMON    = 0x00,
     //! Alias for bold colobot font
-    FONT_COLOBOT_BOLD = FONT_COLOBOT | FONT_BOLD,
+    FONT_COMMON_BOLD = FONT_COMMON | FONT_BOLD,
     //! Alias for italic colobot font
-    FONT_COLOBOT_ITALIC = FONT_COLOBOT | FONT_ITALIC,
+    FONT_COMMON_ITALIC = FONT_COMMON | FONT_ITALIC,
 
-    //! Courier (monospace) font used mainly in code editor (only regular & bold)
-    FONT_COURIER    = 0x01,
-    //! Alias for bold courier font
-    FONT_COURIER_BOLD = FONT_COURIER | FONT_BOLD,
+    //! Studio font used mainly in code editor
+    FONT_STUDIO    = 0x01,
+    //! Alias for bold studio font
+    FONT_STUDIO_BOLD = FONT_STUDIO | FONT_BOLD,
+    //! Alias for italic studio font (at this point not used anywhere)
+    FONT_STUDIO_ITALIC = FONT_STUDIO | FONT_ITALIC,
 
-    // 0x02 left for possible another font
+    //! SatCom font used for interface (currently bold and italic wariants aren't used anywhere)
+    FONT_SATCOM = 0x02,
+    //! Alias for bold satcom font
+    FONT_SATCOM_BOLD = FONT_SATCOM | FONT_BOLD,
+    //! Alias for italic satcom font
+    FONT_SATCOM_ITALIC = FONT_SATCOM | FONT_ITALIC,
 
     //! Pseudo-font loaded from textures for buttons, icons, etc.
     FONT_BUTTON     = 0x03,
@@ -249,6 +256,8 @@ public:
 
     //! Flushes cached textures
     void        FlushCache();
+    //! Try to load new font files
+    bool        ReloadFonts();
 
     //@{
     //! Tab size management
@@ -330,6 +339,8 @@ protected:
     void        StringToUTFCharList(const std::string &text, std::vector<UTF8Char> &chars);
     void        StringToUTFCharList(const std::string &text, std::vector<UTF8Char> &chars, std::vector<FontMetaChar>::iterator format, std::vector<FontMetaChar>::iterator end);
 
+    int GetCharSizeAt(Gfx::FontType font, const std::string& text, unsigned int index) const;
+
 protected:
     CEngine*       m_engine;
     CDevice*       m_device;
@@ -344,6 +355,9 @@ protected:
     FontType     m_lastFontType;
     int          m_lastFontSize;
     CachedFont*  m_lastCachedFont;
+
+    class CQuadBatch;
+    std::unique_ptr<CQuadBatch> m_quadBatch;
 };
 
 
