@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -25,10 +25,11 @@
 // TODO: I'm not sure about "challenges" + "custom". It may be messing things up already right now.
 const std::map<LevelCategory, std::string> CATEGORY_DIR_MAP = {
     { LevelCategory::Missions,     "missions"     },
-    { LevelCategory::FreeGame,     "freemissions" },
+    { LevelCategory::FreeGame,     "free"         },
     { LevelCategory::Exercises,    "exercises"    },
     { LevelCategory::Challenges,   "challenges"   },
     { LevelCategory::CodeBattles,  "battles"      },
+    { LevelCategory::GamePlus,     "plus"         },
     { LevelCategory::CustomLevels, "custom"       },
 };
 
