@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -31,6 +31,8 @@
 #include "graphics/engine/pyro_type.h"
 #include "graphics/engine/water.h"
 
+#include "level/scoreboard.h"
+
 #include "math/point.h"
 
 #include "object/drive_type.h"
@@ -85,6 +87,7 @@ public:
     Gfx::EngineObjectType AsTerrainType();
     int AsBuildFlag();
     int AsResearchFlag();
+    CScoreboard::SortType AsSortType();
     Gfx::PyroType AsPyroType();
     Gfx::CameraType AsCameraType();
     MissionType AsMissionType();
@@ -108,6 +111,7 @@ public:
     Gfx::EngineObjectType AsTerrainType(Gfx::EngineObjectType def);
     int AsBuildFlag(int def);
     int AsResearchFlag(int def);
+    CScoreboard::SortType AsSortType(CScoreboard::SortType def);
     Gfx::PyroType AsPyroType(Gfx::PyroType def);
     Gfx::CameraType AsCameraType(Gfx::CameraType def);
     MissionType AsMissionType(MissionType def);
@@ -139,6 +143,7 @@ private:
     Gfx::EngineObjectType ToTerrainType(std::string value);
     int ToBuildFlag(std::string value);
     int ToResearchFlag(std::string value);
+    CScoreboard::SortType ToSortType(std::string value);
     Gfx::PyroType ToPyroType(std::string value);
     Gfx::CameraType ToCameraType(std::string value);
     MissionType ToMissionType(std::string value);
