@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -55,6 +55,7 @@ enum Phase
     PHASE_APPERANCE,
     PHASE_MAIN_MENU,
     PHASE_LEVEL_LIST,
+    PHASE_MOD_LIST,
     PHASE_SIMUL,
     PHASE_SETUPd,
     PHASE_SETUPg,
@@ -150,6 +151,12 @@ struct MinMax
     int max = -1;
 };
 
+struct Viewpoint
+{
+    Math::Vector    eye{};
+    Math::Vector    look{};
+    int             button = 13; // 13 is the camera button
+};
 
 const int SATCOM_HUSTON     = 0;
 const int SATCOM_SAT        = 1;
@@ -260,6 +267,8 @@ public:
     const std::string& GetScriptName();
     const std::string& GetScriptFile();
     bool        GetTrainerPilot();
+    bool        GetPlusTrainer();
+    bool        GetPlusExplorer();
     bool        GetFixScene();
     bool        GetShowSoluce();
     bool        GetSceneSoluce();
@@ -302,8 +311,8 @@ public:
 
     void        SaveAllScript();
     void        SaveOneScript(CObject *obj);
-    bool        SaveFileStack(CObject *obj, FILE *file, int objRank);
-    bool        ReadFileStack(CObject *obj, FILE *file, int objRank);
+    bool        SaveFileStack(CObject *obj, std::ostream &ostr);
+    bool        ReadFileStack(CObject *obj, std::istream &istr);
 
     //! Return list of scripts to load to robot created in BotFactory
     std::vector<std::string> GetNewScriptNames(ObjectType type);
@@ -461,6 +470,11 @@ public:
     //! Returns global magnifyDamage setting
     float       GetGlobalMagnifyDamage();
 
+    //! Returns global NuclearCell capacity Setting
+    float       GetGlobalNuclearCapacity();
+    //! Returns global PowerCell capacity setting
+    float       GetGlobalCellCapacity();
+
     void        StartDetectEffect(COldObject* object, CObject* target);
 
     //! Enable crash sphere debug rendering
@@ -528,7 +542,7 @@ protected:
     void        UpdateDebugCrashSpheres();
 
     //! Adds element to the beginning of command history
-    void        PushToCommandHistory(std::string obj);
+    void        PushToCommandHistory(std::string cmd);
     //! Returns next/previous element from command history and updates index
     //@{
     std::string    GetNextFromCommandHistory();
@@ -643,6 +657,9 @@ protected:
 
     float           m_globalMagnifyDamage = 0.0f;
 
+    float           m_globalNuclearCapacity = 10.0f;
+    float           m_globalCellCapacity = 1.0f;
+
     bool            m_exitAfterMission = false;
 
     bool            m_codeBattleInit = false;
@@ -667,6 +684,8 @@ protected:
     //! If true, the mission ends immediately after completing the requirements without requiring SpaceShip takeoff
     bool            m_endTakeImmediat = false;
     long            m_endTakeResearch = 0;
+    float           m_endTakeTimeout = -1.0f;
+    bool            m_endTakeTeamImmediateWin = false;
     float           m_endTakeWinDelay = 0.0f;
     float           m_endTakeLostDelay = 0.0f;
     //! Set to true for teams that have already finished
@@ -717,4 +736,7 @@ protected:
     std::deque<std::string> m_commandHistory;
     //! Index of currently selected element in command history
     int             m_commandHistoryIndex;
+
+    //! Vector of available viewpoints
+    std::vector<Viewpoint> m_viewpoints;
 };
