@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -176,7 +176,7 @@ bool CAutoDestroyer::EventProcess(const Event &event)
             if ( scrap != nullptr )
             {
                 assert(scrap->Implements(ObjectInterfaceType::Destroyable));
-                dynamic_cast<CDestroyableObject*>(scrap)->DestroyObject(DestructionType::Explosion);
+                dynamic_cast<CDestroyableObject&>(*scrap).DestroyObject(DestructionType::Explosion);
             }
             m_bExplo = true;
         }
