@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -27,13 +27,10 @@ class CAutoKid : public CAuto
 {
 public:
     CAutoKid(COldObject* object);
-    ~CAutoKid();
+    ~CAutoKid() override;
 
-    void        DeleteObject(bool all = false);
-
-    void        Init();
-    bool        EventProcess(const Event &event);
-    Error       GetError();
+    void        Init() override;
+    bool        EventProcess(const Event &event) override;
 
 protected:
     float       m_speed = 0.0f;
