@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -429,7 +429,7 @@ bool CMotionAnt::EventFrame(const Event &event)
     assert(m_object->Implements(ObjectInterfaceType::Destroyable));
     if ( dynamic_cast<CDestroyableObject*>(m_object)->GetDying() == DeathType::Burning )  // burning?
     {
-        if ( dynamic_cast<CBaseAlien*>(m_object)->GetFixed() )
+        if ( dynamic_cast<CBaseAlien&>(*m_object).GetFixed() )
         {
             m_actionType = MAS_BURN;
         }
@@ -724,7 +724,7 @@ bool CMotionAnt::EventFrame(const Event &event)
         if ( m_progress >= 1.0f )
         {
             SetAction(-1);
-            dynamic_cast<CBaseAlien*>(m_object)->SetFixed(false);  // moving again
+            dynamic_cast<CBaseAlien&>(*m_object).SetFixed(false);  // moving again
         }
     }
     else
