@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 #include "object/object_create_params.h"
 #include "object/object_interface_type.h"
 #include "object/object_type.h"
+
 #include "object/interface/destroyable_object.h"
 
 #include <map>
@@ -302,6 +303,8 @@ public:
     //@}
 
 private:
+    //! Prevents creation of overcharged power cells
+    float ClampPower(ObjectType type, float power);
     void CleanRemovedObjectsIfNeeded();
 
 private:
