@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -101,10 +101,12 @@ enum ObjectType
     OBJECT_TREE3            = 93,   //!< Tree3
     OBJECT_TREE4            = 94,   //!< Tree4
     OBJECT_TREE5            = 95,   //!< Tree5
-    OBJECT_MOBILEwt         = 100,  //!< PracticeBot
-    OBJECT_MOBILEtt         = 101,  //!< track-trainer (unused)
-    OBJECT_MOBILEft         = 102,  //!< fly-trainer (unused)
-    OBJECT_MOBILEit         = 103,  //!< insect-trainer (unused)
+    OBJECT_MOBILEwt         = 100,  //!< WheeledTrainer (PracticeBot)
+    OBJECT_MOBILEtt         = 101,  //!< TrackedTrainer
+    OBJECT_MOBILEft         = 102,  //!< WingedTrainer
+    OBJECT_MOBILEit         = 103,  //!< LeggedTrainer
+    OBJECT_MOBILErp         = 104,  //!< HeavyTrainer
+    OBJECT_MOBILEst         = 105,  //!< AmphibiousTrainer
     OBJECT_MOBILEwa         = 110,  //!< WheeledGrabber
     OBJECT_MOBILEta         = 111,  //!< TrackedGrabber
     OBJECT_MOBILEfa         = 112,  //!< WingedGrabber
@@ -129,6 +131,11 @@ enum ObjectType
     OBJECT_MOBILEtg         = 211,  //!< TargetBot
     OBJECT_MOBILEdr         = 212,  //!< Scribbler
     OBJECT_CONTROLLER       = 213,  //!< MissionController
+    OBJECT_MOBILEwb         = 220,  //!< WheeledBuilder
+    OBJECT_MOBILEtb         = 221,  //!< TrackedBuilder
+    OBJECT_MOBILEfb         = 222,  //!< WingedBuilder
+    OBJECT_MOBILEib         = 223,  //!< LeggedBuilder
+    OBJECT_MOBILEpr         = 224,  //!< PracticeBot (alias)
     OBJECT_WAYPOINT         = 250,  //!< WayPoint
     OBJECT_FLAGb            = 260,  //!< BlueFlag
     OBJECT_FLAGr            = 261,  //!< RedFlag
@@ -241,3 +248,5 @@ struct ObjectTypeHash
         return std::hash<int>()(t);
     }
 };
+
+bool IsValidObjectTypeId(int id);
