@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -138,6 +138,7 @@ COldObject::COldObject(int id)
     m_virusTime = 0.0f;
     m_lastVirusParticle = 0.0f;
     m_damaging = false;
+    m_damageTime = 0.0f;
     m_dying = DeathType::Alive;
     m_bFlat  = false;
     m_gunGoalV = 0.0f;
@@ -283,13 +284,19 @@ void COldObject::DeleteObject(bool bAll)
         if (m_power != nullptr)
         {
             if (m_power->Implements(ObjectInterfaceType::Old))
-                dynamic_cast<COldObject*>(m_power)->DeleteObject(bAll);
+            {
+                dynamic_cast<COldObject&>(*m_power).SetTransporter(nullptr);
+                dynamic_cast<COldObject&>(*m_power).DeleteObject(bAll);
+            }
             m_power = nullptr;
         }
         if (m_cargo != nullptr)
         {
             if (m_cargo->Implements(ObjectInterfaceType::Old))
-                dynamic_cast<COldObject*>(m_cargo)->DeleteObject(bAll);
+            {
+                dynamic_cast<COldObject&>(*m_cargo).SetTransporter(nullptr);
+                dynamic_cast<COldObject&>(*m_cargo).DeleteObject(bAll);
+            }
             m_cargo = nullptr;
         }
     }
@@ -344,6 +351,7 @@ bool COldObject::DamageObject(DamageType type, float force, CObject* killer)
     assert(!Implements(ObjectInterfaceType::Destroyable) || Implements(ObjectInterfaceType::Shielded) || Implements(ObjectInterfaceType::Fragile));
 
     if ( IsDying() )  return false;
+    if ( Implements(ObjectInterfaceType::Jostleable) ) return false;
 
     if ( m_type == OBJECT_ANT    ||
          m_type == OBJECT_WORM   ||
@@ -356,8 +364,26 @@ bool COldObject::DamageObject(DamageType type, float force, CObject* killer)
     }
     else if ( Implements(ObjectInterfaceType::Fragile) )
     {
-        if ( m_type == OBJECT_BOMB && type != DamageType::Explosive ) return false; // Mine can't be destroyed by shooting
-        if ( m_type == OBJECT_URANIUM ) return false; // UraniumOre is not destroyable (see #777)
+        if ((m_type == OBJECT_BOMB         ||
+             m_type == OBJECT_RUINmobilew1 ||
+             m_type == OBJECT_RUINmobilew2 ||
+             m_type == OBJECT_RUINmobilet1 ||
+             m_type == OBJECT_RUINmobilet2 ||
+             m_type == OBJECT_RUINmobiler1 ||
+             m_type == OBJECT_RUINmobiler2 ||
+             m_type == OBJECT_RUINfactory  ||
+             m_type == OBJECT_RUINdoor     ||
+             m_type == OBJECT_RUINsupport  ||
+             m_type == OBJECT_RUINradar    ||
+             m_type == OBJECT_RUINconvert   ) && type != DamageType::Explosive ) return false; // Mines and ruins can't be destroyed by shooting
+        if ( m_type == OBJECT_URANIUM && (type == DamageType::Fire || type == DamageType::Organic) ) return false; // UraniumOre is not destroyable by shooting or aliens (see #777)
+        if ( m_type == OBJECT_STONE && (type == DamageType::Fire || type == DamageType::Organic) ) return false; // TitaniumOre is not destroyable either
+        // PowerCell, NuclearCell and Titanium are destroyable by shooting, but not by collisions!
+        if ( m_type == OBJECT_METAL && type == DamageType::Collision ) return false;
+        if ( m_type == OBJECT_POWER && type == DamageType::Collision ) return false;
+        if ( m_type == OBJECT_NUCLEAR && type == DamageType::Collision ) return false;
+
+        if ( m_magnifyDamage * m_main->GetGlobalMagnifyDamage() == 0 ) return false; // Don't destroy if magnifyDamage=0
 
         DestroyObject(DestructionType::Explosion, killer);
         return true;
@@ -491,7 +517,12 @@ void COldObject::DestroyObject(DestructionType type, CObject* killer)
                   m_type == OBJECT_SAFE     ||
                   m_type == OBJECT_HUSTON   ||
                   m_type == OBJECT_START    ||
-                  m_type == OBJECT_END      )  // building?
+                  m_type == OBJECT_END      ||
+                  m_type == OBJECT_RUINfactory ||
+                  m_type == OBJECT_RUINdoor    ||
+                  m_type == OBJECT_RUINsupport ||
+                  m_type == OBJECT_RUINradar   ||
+                  m_type == OBJECT_RUINconvert  )  // building?
         {
             pyroType = Gfx::PT_FRAGT;
         }
@@ -539,6 +570,11 @@ void COldObject::DestroyObject(DestructionType type, CObject* killer)
     {
         pyroType = Gfx::PT_WPCHECK;
     }
+    else if ( type == DestructionType::Squash )
+    {
+        pyroType = Gfx::PT_SQUASH;
+        DeleteAllCrashSpheres();
+    }
     assert(pyroType != Gfx::PT_NULL);
     if (pyroType == Gfx::PT_FRAGT ||
         pyroType == Gfx::PT_FRAGO ||
@@ -676,12 +712,15 @@ void COldObject::SetType(ObjectType type)
     m_type = type;
     m_name = GetObjectName(m_type);
 
+    SetSelectable(IsSelectableByDefault(m_type));
+
     // TODO: Temporary hack
     if ( m_type == OBJECT_MOBILEfa || // WingedGrabber
+         m_type == OBJECT_MOBILEfb || // WingedBuilder
          m_type == OBJECT_MOBILEfs || // WingedSniffer
          m_type == OBJECT_MOBILEfc || // WingedShooter
          m_type == OBJECT_MOBILEfi || // WingedOrgaShooter
-         m_type == OBJECT_MOBILEft || // winged PracticeBot (unused)
+         m_type == OBJECT_MOBILEft || // WingedTrainer
          m_type == OBJECT_HUMAN    || // Me
          m_type == OBJECT_TECH     || // Tech
          m_type == OBJECT_CONTROLLER)
@@ -705,6 +744,10 @@ void COldObject::SetType(ObjectType type)
         m_type == OBJECT_MOBILEta ||
         m_type == OBJECT_MOBILEwa ||
         m_type == OBJECT_MOBILEia ||
+        m_type == OBJECT_MOBILEfb ||
+        m_type == OBJECT_MOBILEtb ||
+        m_type == OBJECT_MOBILEwb ||
+        m_type == OBJECT_MOBILEib ||
         m_type == OBJECT_MOBILEfc ||
         m_type == OBJECT_MOBILEtc ||
         m_type == OBJECT_MOBILEwc ||
@@ -727,6 +770,8 @@ void COldObject::SetType(ObjectType type)
         m_type == OBJECT_MOBILEtt ||
         m_type == OBJECT_MOBILEwt ||
         m_type == OBJECT_MOBILEit ||
+        m_type == OBJECT_MOBILErp ||
+        m_type == OBJECT_MOBILEst ||
         m_type == OBJECT_TOWER    ||
         m_type == OBJECT_RESEARCH ||
         m_type == OBJECT_ENERGY   ||
@@ -767,6 +812,10 @@ void COldObject::SetType(ObjectType type)
          m_type == OBJECT_MOBILEta ||
          m_type == OBJECT_MOBILEwa ||
          m_type == OBJECT_MOBILEia ||
+         m_type == OBJECT_MOBILEfb ||
+         m_type == OBJECT_MOBILEtb ||
+         m_type == OBJECT_MOBILEwb ||
+         m_type == OBJECT_MOBILEib ||
          m_type == OBJECT_MOBILEfc ||
          m_type == OBJECT_MOBILEtc ||
          m_type == OBJECT_MOBILEwc ||
@@ -788,6 +837,8 @@ void COldObject::SetType(ObjectType type)
          m_type == OBJECT_MOBILEtt ||
          m_type == OBJECT_MOBILEwt ||
          m_type == OBJECT_MOBILEit ||
+         m_type == OBJECT_MOBILErp ||
+         m_type == OBJECT_MOBILEst ||
          m_type == OBJECT_FACTORY  ||
          m_type == OBJECT_REPAIR   ||
          m_type == OBJECT_DESTROYER||
@@ -817,6 +868,38 @@ void COldObject::SetType(ObjectType type)
         m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Fragile)] = false;
         m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Shielded)] = false;
     }
+    else if (m_type == OBJECT_RUINmobilew1 ||
+             m_type == OBJECT_RUINmobilew2 ||
+             m_type == OBJECT_RUINmobilet1 ||
+             m_type == OBJECT_RUINmobilet2 ||
+             m_type == OBJECT_RUINmobiler1 ||
+             m_type == OBJECT_RUINmobiler2 ||
+             m_type == OBJECT_RUINfactory  ||
+             m_type == OBJECT_RUINdoor     ||
+             m_type == OBJECT_RUINsupport  ||
+             m_type == OBJECT_RUINradar    ||
+             m_type == OBJECT_RUINconvert   )
+    {
+        m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Damageable)] = true;
+        m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Destroyable)] = true;
+        m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Fragile)] = true;
+        m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Shielded)] = false;
+    }
+    else if (m_type == OBJECT_PLANT0  ||
+             m_type == OBJECT_PLANT1  ||
+             m_type == OBJECT_PLANT2  ||
+             m_type == OBJECT_PLANT3  ||
+             m_type == OBJECT_PLANT4  ||
+             m_type == OBJECT_PLANT15 ||
+             m_type == OBJECT_PLANT16 ||
+             m_type == OBJECT_PLANT17 ||
+             m_type == OBJECT_PLANT18 )
+    {
+        m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Damageable)] = true;
+        m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Destroyable)] = true;
+        m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Fragile)] = true;
+        m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Shielded)] = false;
+    }
     else
     {
         m_implementedInterfaces[static_cast<int>(ObjectInterfaceType::Damageable)] = false;
@@ -856,6 +939,10 @@ void COldObject::SetType(ObjectType type)
         m_type == OBJECT_MOBILEta ||
         m_type == OBJECT_MOBILEwa ||
         m_type == OBJECT_MOBILEia ||
+        m_type == OBJECT_MOBILEfb ||
+        m_type == OBJECT_MOBILEtb ||
+        m_type == OBJECT_MOBILEwb ||
+        m_type == OBJECT_MOBILEib ||
         m_type == OBJECT_MOBILEfc ||
         m_type == OBJECT_MOBILEtc ||
         m_type == OBJECT_MOBILEwc ||
@@ -877,6 +964,9 @@ void COldObject::SetType(ObjectType type)
         m_type == OBJECT_MOBILEtt ||
         m_type == OBJECT_MOBILEwt ||
         m_type == OBJECT_MOBILEit ||
+        m_type == OBJECT_MOBILErp ||
+        m_type == OBJECT_MOBILEst ||
+        m_type == OBJECT_MOBILEtg ||
         m_type == OBJECT_MOBILEdr ||
         m_type == OBJECT_APOLLO2  ||
         m_type == OBJECT_BASE     ||
@@ -1081,6 +1171,10 @@ void COldObject::Read(CLevelParserLine* line)
     {
         SetRange(line->GetParam("range")->AsFloat(30.0f));
     }
+    if (Implements(ObjectInterfaceType::Fragile))
+    {
+        SetMagnifyDamage(line->GetParam("magnifyDamage")->AsFloat(1.0f)); // TODO: This is a temporary hack for now - CFragileObject doesn't have SetMagnifyDamage ~krzys_h
+    }
     if (Implements(ObjectInterfaceType::Shielded))
     {
         SetShield(line->GetParam("shield")->AsFloat(1.0f));
@@ -1201,7 +1295,7 @@ int COldObject::SearchDescendant(int parent, int n)
 
 void COldObject::TransformCrashSphere(Math::Sphere& crashSphere)
 {
-    crashSphere.radius *= GetScaleX();
+    if(!Implements(ObjectInterfaceType::Jostleable)) crashSphere.radius *= GetScaleX();
 
     // Returns to the sphere collisions,
     // which ignores the tilt of the vehicle.
@@ -1533,6 +1627,11 @@ bool COldObject::GetTrainer()
     return m_bTrainer;
 }
 
+bool COldObject::GetPlusTrainer()
+{
+    return m_main->GetPlusTrainer();
+}
+
 void COldObject::SetToy(bool bEnable)
 {
     m_bToy = bEnable;
@@ -2277,7 +2376,8 @@ void COldObject::AdjustCamera(Math::Vector &eye, float &dirH, float &dirV,
     }
     else if ( m_type == OBJECT_MOBILErt ||
               m_type == OBJECT_MOBILErr ||
-              m_type == OBJECT_MOBILErs )
+              m_type == OBJECT_MOBILErs ||
+              m_type == OBJECT_MOBILErp )
     {
         eye.x = -1.1f;  // on the cap
         eye.y =  7.9f;
@@ -2319,7 +2419,8 @@ void COldObject::AdjustCamera(Math::Vector &eye, float &dirH, float &dirV,
         eye.y = 11.0f;
         eye.z =  0.0f;
     }
-    else if ( m_type == OBJECT_MOBILEsa )
+    else if ( m_type == OBJECT_MOBILEsa ||
+              m_type == OBJECT_MOBILEst )
     {
         eye.x =  3.0f;
         eye.y =  4.5f;
@@ -2404,7 +2505,7 @@ float COldObject::GetAbsTime()
 
 float COldObject::GetCapacity()
 {
-    return m_type == OBJECT_ATOMIC ? 10.0f : 1.0f;
+    return m_type == OBJECT_ATOMIC ? m_main->GetGlobalNuclearCapacity() : m_main->GetGlobalCellCapacity() ;
 }
 
 bool COldObject::IsRechargeable()
@@ -2724,7 +2825,11 @@ void COldObject::SetGunGoalV(float gunGoal)
     if ( m_type == OBJECT_MOBILEfc ||
          m_type == OBJECT_MOBILEtc ||
          m_type == OBJECT_MOBILEwc ||
-         m_type == OBJECT_MOBILEic )  // fireball?
+         m_type == OBJECT_MOBILEic ||
+         m_type == OBJECT_MOBILEfb ||
+         m_type == OBJECT_MOBILEtb ||
+         m_type == OBJECT_MOBILEwb ||
+         m_type == OBJECT_MOBILEib)  // fireball?
     {
         if ( gunGoal >  10.0f*Math::PI/180.0f )  gunGoal =  10.0f*Math::PI/180.0f;
         if ( gunGoal < -20.0f*Math::PI/180.0f )  gunGoal = -20.0f*Math::PI/180.0f;
@@ -2758,7 +2863,11 @@ void COldObject::SetGunGoalH(float gunGoal)
     if ( m_type == OBJECT_MOBILEfc ||
          m_type == OBJECT_MOBILEtc ||
          m_type == OBJECT_MOBILEwc ||
-         m_type == OBJECT_MOBILEic )  // fireball?
+         m_type == OBJECT_MOBILEic ||
+         m_type == OBJECT_MOBILEfb ||
+         m_type == OBJECT_MOBILEtb ||
+         m_type == OBJECT_MOBILEwb ||
+         m_type == OBJECT_MOBILEib)  // fireball?
     {
         if ( gunGoal >  40.0f*Math::PI/180.0f )  gunGoal =  40.0f*Math::PI/180.0f;
         if ( gunGoal < -40.0f*Math::PI/180.0f )  gunGoal = -40.0f*Math::PI/180.0f;
@@ -2832,6 +2941,10 @@ void COldObject::CreateSelectParticle()
              m_type == OBJECT_MOBILEta ||
              m_type == OBJECT_MOBILEwa ||
              m_type == OBJECT_MOBILEia ||
+             m_type == OBJECT_MOBILEfb ||
+             m_type == OBJECT_MOBILEtb ||
+             m_type == OBJECT_MOBILEwb ||
+             m_type == OBJECT_MOBILEib ||
              m_type == OBJECT_MOBILEfc ||
              m_type == OBJECT_MOBILEtc ||
              m_type == OBJECT_MOBILEwc ||
@@ -2854,6 +2967,8 @@ void COldObject::CreateSelectParticle()
              m_type == OBJECT_MOBILEtt ||
              m_type == OBJECT_MOBILEwt ||
              m_type == OBJECT_MOBILEit ||
+             m_type == OBJECT_MOBILErp ||
+             m_type == OBJECT_MOBILEst ||
              m_type == OBJECT_MOBILEdr )  // vehicle?
         {
             pos = Math::Vector(0.0f, 0.0f, 0.0f);
@@ -2890,24 +3005,16 @@ void COldObject::UpdateSelectParticle()
     if ( m_type == OBJECT_MOBILErt ||
          m_type == OBJECT_MOBILErc ||
          m_type == OBJECT_MOBILErr ||
-         m_type == OBJECT_MOBILErs )  // large caterpillars?
+         m_type == OBJECT_MOBILErs ||
+         m_type == OBJECT_MOBILErp )  // large caterpillars?
     {
         pos[0] = Math::Vector(4.2f, 2.8f,  1.5f);
         pos[1] = Math::Vector(4.2f, 2.8f, -1.5f);
         dim[0].x = 1.5f;
         dim[1].x = 1.5f;
     }
-    else if ( m_type == OBJECT_MOBILEwt ||
-              m_type == OBJECT_MOBILEtt ||
-              m_type == OBJECT_MOBILEft ||
-              m_type == OBJECT_MOBILEit )  // trainer ?
-    {
-        pos[0] = Math::Vector(4.2f, 2.5f,  1.2f);
-        pos[1] = Math::Vector(4.2f, 2.5f, -1.2f);
-        dim[0].x = 1.5f;
-        dim[1].x = 1.5f;
-    }
-    else if ( m_type == OBJECT_MOBILEsa )  // submarine?
+    else if ( m_type == OBJECT_MOBILEsa ||
+              m_type == OBJECT_MOBILEst )  // submarine?
     {
         pos[0] = Math::Vector(3.6f, 4.0f,  2.0f);
         pos[1] = Math::Vector(3.6f, 4.0f, -2.0f);
@@ -2922,6 +3029,17 @@ void COldObject::UpdateSelectParticle()
         pos[0] = Math::Vector(4.9f, 3.5f,  2.5f);
         pos[1] = Math::Vector(4.9f, 3.5f, -2.5f);
     }
+    else if ( m_type == OBJECT_MOBILEwt ||
+              m_type == OBJECT_MOBILEtt ||
+              m_type == OBJECT_MOBILEft ||
+              m_type == OBJECT_MOBILEit ||
+              GetTrainer())                // trainer ?
+    {
+        pos[0] = Math::Vector(4.2f, 2.5f,  1.2f);
+        pos[1] = Math::Vector(4.2f, 2.5f, -1.2f);
+        dim[0].x = 1.5f;
+        dim[1].x = 1.5f;
+    }
     else
     {
         pos[0] = Math::Vector(4.2f, 2.5f,  1.5f);
@@ -2929,57 +3047,72 @@ void COldObject::UpdateSelectParticle()
     }
 
     // Red back lens
-    if ( m_type == OBJECT_MOBILEfa ||
-         m_type == OBJECT_MOBILEfc ||
-         m_type == OBJECT_MOBILEfi ||
-         m_type == OBJECT_MOBILEfs ||
-         m_type == OBJECT_MOBILEft )  // flying?
+    if ( m_type == OBJECT_MOBILEwt ||
+         m_type == OBJECT_MOBILEtt ||
+         m_type == OBJECT_MOBILEft ||
+         m_type == OBJECT_MOBILEit ||
+         GetTrainer())               // trainer?
+    {
+        pos[2] = Math::Vector(-4.0f, 2.5f,  2.2f);
+        pos[3] = Math::Vector(-4.0f, 2.5f, -2.2f);
+    }
+    else if ( m_type == OBJECT_MOBILEfa ||
+              m_type == OBJECT_MOBILEfb ||
+              m_type == OBJECT_MOBILEfc ||
+              m_type == OBJECT_MOBILEfi ||
+              m_type == OBJECT_MOBILEfs )  // flying?
     {
         pos[2] = Math::Vector(-4.0f, 3.1f,  4.5f);
         pos[3] = Math::Vector(-4.0f, 3.1f, -4.5f);
         dim[2].x = 0.6f;
         dim[3].x = 0.6f;
     }
-    if ( m_type == OBJECT_MOBILEwa ||
-         m_type == OBJECT_MOBILEwc ||
-         m_type == OBJECT_MOBILEwi ||
-         m_type == OBJECT_MOBILEws )  // wheels?
+    else if ( m_type == OBJECT_MOBILEwa ||
+              m_type == OBJECT_MOBILEwb ||
+              m_type == OBJECT_MOBILEwc ||
+              m_type == OBJECT_MOBILEwi ||
+              m_type == OBJECT_MOBILEws )  // wheels?
     {
         pos[2] = Math::Vector(-4.5f, 2.7f,  2.8f);
         pos[3] = Math::Vector(-4.5f, 2.7f, -2.8f);
     }
-    if ( m_type == OBJECT_MOBILEwt )  // wheels?
-    {
-        pos[2] = Math::Vector(-4.0f, 2.5f,  2.2f);
-        pos[3] = Math::Vector(-4.0f, 2.5f, -2.2f);
-    }
-    if ( m_type == OBJECT_MOBILEia ||
-         m_type == OBJECT_MOBILEic ||
-         m_type == OBJECT_MOBILEii ||
-         m_type == OBJECT_MOBILEis ||
-         m_type == OBJECT_MOBILEit )  // legs?
+    else if ( m_type == OBJECT_MOBILEia ||
+              m_type == OBJECT_MOBILEib ||
+              m_type == OBJECT_MOBILEic ||
+              m_type == OBJECT_MOBILEii ||
+              m_type == OBJECT_MOBILEis )  // legs?
     {
         pos[2] = Math::Vector(-4.5f, 2.7f,  2.8f);
         pos[3] = Math::Vector(-4.5f, 2.7f, -2.8f);
     }
-    if ( m_type == OBJECT_MOBILEta ||
-         m_type == OBJECT_MOBILEtc ||
-         m_type == OBJECT_MOBILEti ||
-         m_type == OBJECT_MOBILEts ||
-         m_type == OBJECT_MOBILEtt )  // caterpillars?
+    else if ( m_type == OBJECT_MOBILEta ||
+              m_type == OBJECT_MOBILEtb ||
+              m_type == OBJECT_MOBILEtc ||
+              m_type == OBJECT_MOBILEti ||
+              m_type == OBJECT_MOBILEts )  // caterpillars?
     {
         pos[2] = Math::Vector(-3.6f, 4.2f,  3.0f);
         pos[3] = Math::Vector(-3.6f, 4.2f, -3.0f);
     }
-    if ( m_type == OBJECT_MOBILErt ||
-         m_type == OBJECT_MOBILErc ||
-         m_type == OBJECT_MOBILErr ||
-         m_type == OBJECT_MOBILErs )  // large caterpillars?
+    else if ( m_type == OBJECT_MOBILErt ||
+              m_type == OBJECT_MOBILErc ||
+              m_type == OBJECT_MOBILErr ||
+              m_type == OBJECT_MOBILErs )  // large caterpillars?
     {
         pos[2] = Math::Vector(-5.0f, 5.2f,  2.5f);
         pos[3] = Math::Vector(-5.0f, 5.2f, -2.5f);
     }
-    if ( m_type == OBJECT_MOBILEsa )  // submarine?
+    if ( m_type == OBJECT_MOBILErp || ( GetTrainer() &&
+       ( m_type == OBJECT_MOBILErt ||
+         m_type == OBJECT_MOBILErc ||
+         m_type == OBJECT_MOBILErr ||
+         m_type == OBJECT_MOBILErs)))  // large caterpillars (trainer)?
+    {
+        pos[2] = Math::Vector(-4.6f, 5.2f,  2.6f);
+        pos[3] = Math::Vector(-4.6f, 5.2f, -2.6f);
+    }
+    if ( m_type == OBJECT_MOBILEsa ||
+         m_type == OBJECT_MOBILEst )  // submarine?
     {
         pos[2] = Math::Vector(-3.6f, 4.0f,  2.0f);
         pos[3] = Math::Vector(-3.6f, 4.0f, -2.0f);
@@ -3198,6 +3331,10 @@ float COldObject::GetLightningHitProbability()
          m_type == OBJECT_MOBILEta ||
          m_type == OBJECT_MOBILEwa ||
          m_type == OBJECT_MOBILEia ||
+         m_type == OBJECT_MOBILEfb ||
+         m_type == OBJECT_MOBILEtb ||
+         m_type == OBJECT_MOBILEwb ||
+         m_type == OBJECT_MOBILEib ||
          m_type == OBJECT_MOBILEfc ||
          m_type == OBJECT_MOBILEtc ||
          m_type == OBJECT_MOBILEwc ||
@@ -3219,6 +3356,9 @@ float COldObject::GetLightningHitProbability()
          m_type == OBJECT_MOBILEtt ||
          m_type == OBJECT_MOBILEwt ||
          m_type == OBJECT_MOBILEit ||
+         m_type == OBJECT_MOBILErp ||
+         m_type == OBJECT_MOBILEst ||
+         m_type == OBJECT_MOBILEtg ||
          m_type == OBJECT_MOBILEdr )  // robot?
     {
         return 0.5f;
@@ -3232,8 +3372,7 @@ bool COldObject::IsSelectableByDefault(ObjectType type)
          type == OBJECT_ANT      ||
          type == OBJECT_SPIDER   ||
          type == OBJECT_BEE      ||
-         type == OBJECT_WORM     ||
-         type == OBJECT_MOBILEtg )
+         type == OBJECT_WORM     )
     {
         return false;
     }
