@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -165,6 +165,7 @@ public:
 
     void        SetTrainer(bool bEnable) override;
     bool        GetTrainer() override;
+    bool        GetPlusTrainer();
 
     void        SetToy(bool bEnable);
     bool        GetToy();
@@ -237,7 +238,7 @@ public:
     void        SetMagnifyDamage(float factor) override;
     float       GetMagnifyDamage() override;
 
-    void        SetDamaging(bool damaging);
+    void        SetDamaging(bool damaging) override;
     bool        IsDamaging()  override;
 
     void        SetDying(DeathType deathType) override;
