@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -147,7 +147,7 @@ protected:
     int             m_bmTotal = 0;      // number of points in m_bmPoints
     int             m_bmIndex = 0;      // index in m_bmPoints
     Math::Vector        m_bmPoints[MAXPOINTS+2];
-    char            m_bmIter[MAXPOINTS+2] = {};
+    signed char     m_bmIter[MAXPOINTS+2] = {};
     int             m_bmIterCounter = 0;
     CObject*        m_bmCargoObject = nullptr;
     float           m_bmFinalMove = 0.0f;  // final advance distance
