@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -82,7 +82,12 @@ const char* GetObjectName(ObjectType type)
     if ( type == OBJECT_MARKKEYb    )  return "KeyBSpot";
     if ( type == OBJECT_MARKKEYc    )  return "KeyCSpot";
     if ( type == OBJECT_MARKKEYd    )  return "KeyDSpot";
-    if ( type == OBJECT_MOBILEwt    )  return "PracticeBot";
+    if ( type == OBJECT_MOBILEwt    )  return "WheeledTrainer";
+    if ( type == OBJECT_MOBILEtt    )  return "TrackedTrainer";
+    if ( type == OBJECT_MOBILEft    )  return "WingedTrainer";
+    if ( type == OBJECT_MOBILEit    )  return "LeggedTrainer";
+    if ( type == OBJECT_MOBILErp    )  return "HeavyTrainer";
+    if ( type == OBJECT_MOBILEst    )  return "AmphibiousTrainer";
     if ( type == OBJECT_MOBILEwa    )  return "WheeledGrabber";
     if ( type == OBJECT_MOBILEta    )  return "TrackedGrabber";
     if ( type == OBJECT_MOBILEfa    )  return "WingedGrabber";
@@ -99,6 +104,10 @@ const char* GetObjectName(ObjectType type)
     if ( type == OBJECT_MOBILEts    )  return "TrackedSniffer";
     if ( type == OBJECT_MOBILEfs    )  return "WingedSniffer";
     if ( type == OBJECT_MOBILEis    )  return "LeggedSniffer";
+    if ( type == OBJECT_MOBILEwb    )  return "WheeledBuilder";
+    if ( type == OBJECT_MOBILEtb    )  return "TrackedBuilder";
+    if ( type == OBJECT_MOBILEfb    )  return "WingedBuilder";
+    if ( type == OBJECT_MOBILEib    )  return "LeggedBuilder";
     if ( type == OBJECT_MOBILErt    )  return "Thumper";
     if ( type == OBJECT_MOBILErc    )  return "PhazerShooter";
     if ( type == OBJECT_MOBILErr    )  return "Recycler";
@@ -106,6 +115,7 @@ const char* GetObjectName(ObjectType type)
     if ( type == OBJECT_MOBILEsa    )  return "Subber";
     if ( type == OBJECT_MOBILEtg    )  return "TargetBot";
     if ( type == OBJECT_MOBILEdr    )  return "Scribbler";
+    if ( type == OBJECT_MOBILEpr    )  return "PracticeBot";
     if ( type == OBJECT_HUMAN       )  return "Me";
     if ( type == OBJECT_TECH        )  return "Tech";
     if ( type == OBJECT_MOTHER      )  return "AlienQueen";
@@ -115,6 +125,12 @@ const char* GetObjectName(ObjectType type)
     if ( type == OBJECT_BEE         )  return "AlienWasp";
     if ( type == OBJECT_WORM        )  return "AlienWorm";
     if ( type == OBJECT_RUINmobilew1)  return "Wreck";
+    if ( type == OBJECT_RUINfactory )  return "Ruin";
+    if ( type == OBJECT_PLANT0      )  return "Bush";
+    if ( type == OBJECT_ROOT5       )  return "GraviPlant";
+    if ( type == OBJECT_QUARTZ0     )  return "Crystal";
+    if ( type == OBJECT_MUSHROOM1   )  return "BrownMushroom";
+    if ( type == OBJECT_MUSHROOM2   )  return "GreenMushroom";
     return "";
 }
 
@@ -196,14 +212,24 @@ std::string GetHelpFilename(ObjectType type)
     if ( type == OBJECT_MOBILEtc    )  helpfile = "object/botfc";
     if ( type == OBJECT_MOBILEfc    )  helpfile = "object/botfj";
     if ( type == OBJECT_MOBILEic    )  helpfile = "object/botfs";
+    if ( type == OBJECT_MOBILEwb    )  helpfile = "object/botbr";
+    if ( type == OBJECT_MOBILEtb    )  helpfile = "object/botbc";
+    if ( type == OBJECT_MOBILEfb    )  helpfile = "object/botbj";
+    if ( type == OBJECT_MOBILEib    )  helpfile = "object/botbs";
     if ( type == OBJECT_MOBILErt    )  helpfile = "object/bottump";
     if ( type == OBJECT_MOBILErc    )  helpfile = "object/botphaz";
     if ( type == OBJECT_MOBILErr    )  helpfile = "object/botrecy";
     if ( type == OBJECT_MOBILErs    )  helpfile = "object/botshld";
     if ( type == OBJECT_MOBILEsa    )  helpfile = "object/botsub";
     if ( type == OBJECT_MOBILEwt    )  helpfile = "object/bottr";
+    if ( type == OBJECT_MOBILEtt    )  helpfile = "object/bottr";
+    if ( type == OBJECT_MOBILEft    )  helpfile = "object/bottr";
+    if ( type == OBJECT_MOBILEit    )  helpfile = "object/bottr";
+    if ( type == OBJECT_MOBILErp    )  helpfile = "object/bottr";
+    if ( type == OBJECT_MOBILEst    )  helpfile = "object/bottr";
     if ( type == OBJECT_MOBILEtg    )  helpfile = "object/bottarg";
     if ( type == OBJECT_MOBILEdr    )  helpfile = "object/botdraw";
+    if ( type == OBJECT_MOBILEpr    )  helpfile = "object/bottr";
     if ( type == OBJECT_APOLLO2     )  helpfile = "object/lrv";
     if ( type == OBJECT_HUMAN       )  helpfile = "object/human";
     if ( type == OBJECT_MOTHER      )  helpfile = "object/mother";
@@ -213,6 +239,12 @@ std::string GetHelpFilename(ObjectType type)
     if ( type == OBJECT_BEE         )  helpfile = "object/wasp";
     if ( type == OBJECT_WORM        )  helpfile = "object/worm";
     if ( type == OBJECT_RUINmobilew1)  helpfile = "object/wreck";
+    if ( type == OBJECT_RUINfactory )  helpfile = "object/ruin";
+    if ( type == OBJECT_PLANT0      )  helpfile = "object/bush";
+    if ( type == OBJECT_ROOT5       )  helpfile = "object/gravi";
+    if ( type == OBJECT_QUARTZ0     )  helpfile = "object/crystal";
+    if ( type == OBJECT_MUSHROOM1   )  helpfile = "object/bromush";
+    if ( type == OBJECT_MUSHROOM2   )  helpfile = "object/gremush";
 
     if (helpfile.empty())
         return "";
@@ -229,6 +261,7 @@ std::string GetHelpFilename(const char *token)
 
     if ( strcmp(token, "if"            ) == 0 )  helpfile = "cbot/if";
     if ( strcmp(token, "else"          ) == 0 )  helpfile = "cbot/if";
+    if ( strcmp(token, "repeat"        ) == 0 )  helpfile = "cbot/repeat";
     if ( strcmp(token, "for"           ) == 0 )  helpfile = "cbot/for";
     if ( strcmp(token, "while"         ) == 0 )  helpfile = "cbot/while";
     if ( strcmp(token, "do"            ) == 0 )  helpfile = "cbot/do";
@@ -236,8 +269,13 @@ std::string GetHelpFilename(const char *token)
     if ( strcmp(token, "continue"      ) == 0 )  helpfile = "cbot/continue";
     if ( strcmp(token, "return"        ) == 0 )  helpfile = "cbot/return";
     if ( strcmp(token, "sizeof"        ) == 0 )  helpfile = "cbot/sizeof";
+    if ( strcmp(token, "byte"          ) == 0 )  helpfile = "cbot/byte";
+    if ( strcmp(token, "short"         ) == 0 )  helpfile = "cbot/short";
+    if ( strcmp(token, "char"          ) == 0 )  helpfile = "cbot/char";
     if ( strcmp(token, "int"           ) == 0 )  helpfile = "cbot/int";
+    if ( strcmp(token, "long"          ) == 0 )  helpfile = "cbot/long";
     if ( strcmp(token, "float"         ) == 0 )  helpfile = "cbot/float";
+    if ( strcmp(token, "double"        ) == 0 )  helpfile = "cbot/double";
     if ( strcmp(token, "bool"          ) == 0 )  helpfile = "cbot/bool";
     if ( strcmp(token, "string"        ) == 0 )  helpfile = "cbot/string";
     if ( strcmp(token, "point"         ) == 0 )  helpfile = "cbot/point";
@@ -265,7 +303,7 @@ std::string GetHelpFilename(const char *token)
     if ( strcmp(token, "trunc"         ) == 0 )  helpfile = "cbot/trunc";
     if ( strcmp(token, "retobject"     ) == 0 )  helpfile = "cbot/retobj";
     if ( strcmp(token, "errmode"       ) == 0 )  helpfile = "cbot/errmode";
-    if ( strcmp(token, "busy"          ) == 0 )  helpfile = "cbot/busy";
+    if ( strcmp(token, "isbusy"          ) == 0 )  helpfile = "cbot/isbusy";
     if ( strcmp(token, "takeoff"       ) == 0 )  helpfile = "cbot/takeoff";
     if ( strcmp(token, "research"      ) == 0 )  helpfile = "cbot/research";
     if ( strcmp(token, "factory"       ) == 0 )  helpfile = "cbot/factory";
@@ -285,6 +323,8 @@ std::string GetHelpFilename(const char *token)
     if ( strcmp(token, "researched"     ) == 0 )  helpfile = "cbot/researched";
     if ( strcmp(token, "buildingenabled") == 0 )  helpfile = "cbot/buildingenabled";
     if ( strcmp(token, "build"         ) == 0 )  helpfile = "cbot/build";
+    if ( strcmp(token, "flag"          ) == 0 )  helpfile = "cbot/flag";
+    if ( strcmp(token, "deflag"        ) == 0 )  helpfile = "cbot/deflag";
     if ( strcmp(token, "wait"          ) == 0 )  helpfile = "cbot/wait";
     if ( strcmp(token, "move"          ) == 0 )  helpfile = "cbot/move";
     if ( strcmp(token, "turn"          ) == 0 )  helpfile = "cbot/turn";
@@ -307,23 +347,24 @@ std::string GetHelpFilename(const char *token)
     if ( strcmp(token, "topo"          ) == 0 )  helpfile = "cbot/topo";
     if ( strcmp(token, "message"       ) == 0 )  helpfile = "cbot/message";
     if ( strcmp(token, "abstime"       ) == 0 )  helpfile = "cbot/abstime";
+    if ( strcmp(token, "Blue"          ) == 0 )  helpfile = "cbot/flag";
+    if ( strcmp(token, "Red"           ) == 0 )  helpfile = "cbot/flag";
+    if ( strcmp(token, "Green"         ) == 0 )  helpfile = "cbot/flag";
+    if ( strcmp(token, "Yellow"        ) == 0 )  helpfile = "cbot/flag";
+    if ( strcmp(token, "Violet"        ) == 0 )  helpfile = "cbot/flag";
     if ( strcmp(token, "BlackArrow"    ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "RedArrow"      ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "White"         ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "Black"         ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "Gray"          ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "LightGray"     ) == 0 )  helpfile = "cbot/pendown";
-    if ( strcmp(token, "Red"           ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "Pink"          ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "Purple"        ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "Orange"        ) == 0 )  helpfile = "cbot/pendown";
-    if ( strcmp(token, "Yellow"        ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "Beige"         ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "Brown"         ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "Skin"          ) == 0 )  helpfile = "cbot/pendown";
-    if ( strcmp(token, "Green"         ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "LightGreen"    ) == 0 )  helpfile = "cbot/pendown";
-    if ( strcmp(token, "Blue"          ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "LightBlue"     ) == 0 )  helpfile = "cbot/pendown";
     if ( strcmp(token, "InFront"       ) == 0 )  helpfile = "cbot/grab";
     if ( strcmp(token, "Behind"        ) == 0 )  helpfile = "cbot/grab";
@@ -372,6 +413,9 @@ std::string GetHelpFilename(const char *token)
     if ( strcmp(token, "synchronized"  ) == 0 )  helpfile = "cbot/synchro";
     if ( strcmp(token, "new"           ) == 0 )  helpfile = "cbot/new";
     if ( strcmp(token, "this"          ) == 0 )  helpfile = "cbot/this";
+    if ( strcmp(token, "switch"        ) == 0 ||
+         strcmp(token, "case"          ) == 0 ||
+         strcmp(token, "default"       ) == 0 )  helpfile = "cbot/switch";
 
     if (helpfile.empty())
         return "";
@@ -385,8 +429,13 @@ std::string GetHelpFilename(const char *token)
 bool IsType(const char *token)
 {
     if ( strcmp(token, "void"   ) == 0 )  return true;
+    if ( strcmp(token, "byte"   ) == 0 )  return true;
+    if ( strcmp(token, "short"  ) == 0 )  return true;
+    if ( strcmp(token, "char"   ) == 0 )  return true;
     if ( strcmp(token, "int"    ) == 0 )  return true;
+    if ( strcmp(token, "long"   ) == 0 )  return true;
     if ( strcmp(token, "float"  ) == 0 )  return true;
+    if ( strcmp(token, "double" ) == 0 )  return true;
     if ( strcmp(token, "bool"   ) == 0 )  return true;
     if ( strcmp(token, "string" ) == 0 )  return true;
     if ( strcmp(token, "point"  ) == 0 )  return true;
@@ -416,7 +465,7 @@ bool IsFunction(const char *token)
     if ( strcmp(token, "trunc"        ) == 0 )  return true;
     if ( strcmp(token, "retobjectbyid") == 0 )  return true;
     if ( strcmp(token, "retobject"    ) == 0 )  return true;
-    if ( strcmp(token, "busy"         ) == 0 )  return true;
+    if ( strcmp(token, "isbusy"         ) == 0 )  return true;
     if ( strcmp(token, "factory"      ) == 0 )  return true;
     if ( strcmp(token, "research"     ) == 0 )  return true;
     if ( strcmp(token, "takeoff"      ) == 0 )  return true;
@@ -437,6 +486,8 @@ bool IsFunction(const char *token)
     if ( strcmp(token, "researched"     ) == 0 )  return true;
     if ( strcmp(token, "buildingenabled") == 0 )  return true;
     if ( strcmp(token, "build"        ) == 0 )  return true;
+    if ( strcmp(token, "flag"         ) == 0 )  return true;
+    if ( strcmp(token, "deflag"       ) == 0 )  return true;
     if ( strcmp(token, "wait"         ) == 0 )  return true;
     if ( strcmp(token, "move"         ) == 0 )  return true;
     if ( strcmp(token, "turn"         ) == 0 )  return true;
@@ -491,11 +542,15 @@ bool IsFunction(const char *token)
 
 const char* GetHelpText(const char *token)
 {
-    if ( strcmp(token, "if"        ) == 0 )  return "if ( condition ) { bloc }";
-    if ( strcmp(token, "else"      ) == 0 )  return "else { bloc }";
+    if ( strcmp(token, "if"        ) == 0 )  return "if ( condition ) { code }";
+    if ( strcmp(token, "else"      ) == 0 )  return "else { code }";
+    if ( strcmp(token, "repeat"    ) == 0 )  return "repeat ( number )";
     if ( strcmp(token, "for"       ) == 0 )  return "for ( before ; condition ; end )";
-    if ( strcmp(token, "while"     ) == 0 )  return "while ( condition ) { bloc }";
-    if ( strcmp(token, "do"        ) == 0 )  return "do { bloc } while ( condition );";
+    if ( strcmp(token, "while"     ) == 0 )  return "while ( condition ) { code }";
+    if ( strcmp(token, "do"        ) == 0 )  return "do { code } while ( condition );";
+    if ( strcmp(token, "switch"    ) == 0 )  return "switch ( value ) { code }";
+    if ( strcmp(token, "case"      ) == 0 )  return "case label: { code }";
+    if ( strcmp(token, "default"   ) == 0 )  return "default: { code } ";
     if ( strcmp(token, "break"     ) == 0 )  return "break;";
     if ( strcmp(token, "continue"  ) == 0 )  return "continue;";
     if ( strcmp(token, "return"    ) == 0 )  return "return;";
@@ -519,11 +574,11 @@ const char* GetHelpText(const char *token)
     if ( strcmp(token, "retobject"    ) == 0 )  return "retobject ( rank );";
     if ( strcmp(token, "retobjectbyid") == 0 )  return "retobjectbyid ( rank );";
     if ( strcmp(token, "progfunc"  ) == 0 )  return "progfunc ( funcname );";
-    if ( strcmp(token, "busy"      ) == 0 )  return "object.busy ( );";
-    if ( strcmp(token, "factory"   ) == 0 )  return "object.factory ( cat, program );";
-    if ( strcmp(token, "research"  ) == 0 )  return "object.research ( type );";
-    if ( strcmp(token, "takeoff"   ) == 0 )  return "object.takeoff ( );";
-    if ( strcmp(token, "destroy"   ) == 0 )  return "object.destroy ( );";
+    if ( strcmp(token, "isbusy"      ) == 0 )  return "isbusy ( object );";
+    if ( strcmp(token, "factory"   ) == 0 )  return "factory ( cat, program, object );";
+    if ( strcmp(token, "research"  ) == 0 )  return "research ( type, object );";
+    if ( strcmp(token, "takeoff"   ) == 0 )  return "takeoff ( object );";
+    if ( strcmp(token, "destroy"   ) == 0 )  return "destroy ( object );";
     if ( strcmp(token, "search"    ) == 0 )  return "search ( cat, pos, min, max, sens, filter );";
     if ( strcmp(token, "searchall" ) == 0 )  return "searchall ( cat, pos, min, max, sens, filter );";
     if ( strcmp(token, "radar"     ) == 0 )  return "radar ( cat, angle, focus, min, max, sens, filter );";
@@ -540,6 +595,8 @@ const char* GetHelpText(const char *token)
     if ( strcmp(token, "researched"     ) == 0 )  return "researched ( research );";
     if ( strcmp(token, "buildingenabled") == 0 )  return "buildingenabled ( category );";
     if ( strcmp(token, "build"     ) == 0 )  return "build ( category );";
+    if ( strcmp(token, "flag"      ) == 0 )  return "flag ( color );";
+    if ( strcmp(token, "deflag"    ) == 0 )  return "deflag ( );";
     if ( strcmp(token, "wait"      ) == 0 )  return "wait ( time );";
     if ( strcmp(token, "move"      ) == 0 )  return "move ( distance );";
     if ( strcmp(token, "turn"      ) == 0 )  return "turn ( angle );";
