@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -479,7 +479,7 @@ bool CScript::GetCursor(int &cursor1, int &cursor2)
 
 // Put of the variables in a list.
 
-void PutList(const std::string& baseName, bool bArray, CBot::CBotVar *var, Ui::CList *list, int &rankList, std::set<CBot::CBotVar*>& previous)
+static void PutList(const std::string& baseName, bool bArray, CBot::CBotVar *var, Ui::CList *list, int &rankList, std::set<CBot::CBotVar*>& previous)
 {
     if ( var == nullptr && !baseName.empty() )
     {
@@ -589,16 +589,17 @@ void CScript::UpdateList(Ui::CList* list)
     list->SetState(Ui::STATE_ENABLE);
 }
 
-// Colorize a string literal with escape sequences also colored
+// Colorize a string or character literal with escape sequences also colored
 
-void HighlightString(Ui::CEdit* edit, const std::string& s, int start)
+static void HighlightString(Ui::CEdit* edit, const std::string& s, int start)
 {
     edit->SetFormat(start, start + 1, Gfx::FONT_HIGHLIGHT_STRING);
 
-    auto it = s.cbegin() + 1;
+    auto it = s.cbegin();
+    char endQuote = *(it++);
 
     ++start;
-    while (it != s.cend() && *it != '\"')
+    while (it != s.cend() && *it != endQuote)
     {
         if (*(it++) != '\\') // not escape sequence
         {
@@ -697,7 +698,7 @@ void CScript::ColorizeScript(Ui::CEdit* edit, int rangeStart, int rangeEnd)
         {
             color = Gfx::FONT_HIGHLIGHT_STRING;
         }
-        else if (type == CBot::TokenTypString) // string literals
+        else if (type == CBot::TokenTypString || type == CBot::TokenTypChar) // string literals and character literals
         {
             HighlightString(edit, token, cursor1);
             bt = bt->GetNext();
@@ -716,7 +717,7 @@ void CScript::ColorizeScript(Ui::CEdit* edit, int rangeStart, int rangeEnd)
 // Returns the index of the start of the token found, or -1.
 
 
-int SearchToken(char* script, const char* token)
+static int SearchToken(char* script, const char* token)
 {
     int     lScript, lToken, i, iFound;
     int     found[100];
@@ -741,7 +742,7 @@ int SearchToken(char* script, const char* token)
 
 // Removes a token in a script.
 
-void DeleteToken(char* script, int pos, int len)
+static void DeleteToken(char* script, int pos, int len)
 {
     while ( true )
     {
@@ -752,7 +753,7 @@ void DeleteToken(char* script, int pos, int len)
 
 // Inserts a token in a script.
 
-void InsertToken(char* script, int pos, const char* token)
+static void InsertToken(char* script, int pos, const char* token)
 {
     int     lScript, lToken, i;
 
@@ -1032,16 +1033,16 @@ bool CScript::WriteScript(const char* filename)
 
 // Reads a stack of script by execution as a file.
 
-bool CScript::ReadStack(FILE *file)
+bool CScript::ReadStack(std::istream &istr)
 {
     int     nb;
 
-    CBot::fRead(&nb, sizeof(int), 1, file);
-    CBot::fRead(&m_ipf, sizeof(int), 1, file);
-    CBot::fRead(&m_errMode, sizeof(int), 1, file);
+    if (!CBot::ReadInt(istr, nb)) return false;
+    if (!CBot::ReadInt(istr, m_ipf)) return false;
+    if (!CBot::ReadInt(istr, m_errMode)) return false;
 
     if (m_botProg == nullptr) return false;
-    if ( !m_botProg->RestoreState(file) )  return false;
+    if (!m_botProg->RestoreState(istr)) return false;
 
     m_bRun = true;
     m_bContinue = false;
@@ -1050,16 +1051,16 @@ bool CScript::ReadStack(FILE *file)
 
 // Writes a stack of script by execution as a file.
 
-bool CScript::WriteStack(FILE *file)
+bool CScript::WriteStack(std::ostream &ostr)
 {
     int     nb;
 
     nb = 2;
-    CBot::fWrite(&nb, sizeof(int), 1, file);
-    CBot::fWrite(&m_ipf, sizeof(int), 1, file);
-    CBot::fWrite(&m_errMode, sizeof(int), 1, file);
+    if (!CBot::WriteInt(ostr, nb)) return false;
+    if (!CBot::WriteInt(ostr, m_ipf)) return false;
+    if (!CBot::WriteInt(ostr, m_errMode)) return false;
 
-    return m_botProg->SaveState(file);
+    return m_botProg->SaveState(ostr);
 }
 
 
