@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 #include "CBot/CBot.h"
 
 #include <memory>
+#include <limits>
 #include <string>
 #include <boost/optional.hpp>
 
@@ -88,8 +89,8 @@ public:
     bool        SendScript(const char* text);
     bool        ReadScript(const char* filename);
     bool        WriteScript(const char* filename);
-    bool        ReadStack(FILE *file);
-    bool        WriteStack(FILE *file);
+    bool        ReadStack(std::istream &istr);
+    bool        WriteStack(std::ostream &ostr);
     bool        Compare(CScript* other);
 
     void        SetFilename(const std::string &filename);
