@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -100,6 +100,8 @@ CObjectInterface::CObjectInterface(COldObject* object)
     m_manipStyle   = EVENT_OBJECT_MFRONT;
 
     m_selScript = 0;
+
+    m_buildInterface = false;
 }
 
 // Object's destructor.
@@ -534,9 +536,9 @@ bool CObjectInterface::EventProcess(const Event &event)
         {
             err = m_taskExecutor->StartTaskBuild(OBJECT_INFO);
         }
-        if ( action == EVENT_OBJECT_BDESTROYER )
+        if ( action == EVENT_OBJECT_BSAFE )
         {
-            err = m_taskExecutor->StartTaskBuild(OBJECT_DESTROYER);
+            err = m_taskExecutor->StartTaskBuild(OBJECT_SAFE);
         }
 
         if ( action == EVENT_OBJECT_GFLAT )
@@ -593,7 +595,7 @@ bool CObjectInterface::EventProcess(const Event &event)
                 ps = static_cast< CSlider* >(pw->SearchControl(EVENT_OBJECT_DIMSHIELD));
                 if ( ps != nullptr )
                 {
-                    dynamic_cast<CShielder*>(m_object)->SetShieldRadius((ps->GetVisibleValue()-(RADIUS_SHIELD_MIN/g_unit))/((RADIUS_SHIELD_MAX-RADIUS_SHIELD_MIN)/g_unit));
+                    dynamic_cast<CShielder&>(*m_object).SetShieldRadius((ps->GetVisibleValue()-(RADIUS_SHIELD_MIN/g_unit))/((RADIUS_SHIELD_MAX-RADIUS_SHIELD_MIN)/g_unit));
                 }
             }
         }
@@ -621,6 +623,12 @@ bool CObjectInterface::EventProcess(const Event &event)
             err = m_taskExecutor->StartTaskSpiderExplo();
         }
 
+        if ( action == EVENT_OBJECT_BUILD )
+        {
+            m_buildInterface = !m_buildInterface;
+            UpdateInterface();
+        }
+
         if ( action == EVENT_OBJECT_PEN0 )  // up
         {
             err = m_taskExecutor->StartTaskPen(false);
@@ -841,6 +849,10 @@ bool CObjectInterface::CreateInterface(bool bSelect)
          type == OBJECT_MOBILEta ||
          type == OBJECT_MOBILEwa ||
          type == OBJECT_MOBILEia ||
+         type == OBJECT_MOBILEfb ||
+         type == OBJECT_MOBILEtb ||
+         type == OBJECT_MOBILEwb ||
+         type == OBJECT_MOBILEib ||
          type == OBJECT_MOBILEfc ||
          type == OBJECT_MOBILEtc ||
          type == OBJECT_MOBILEwc ||
@@ -863,6 +875,8 @@ bool CObjectInterface::CreateInterface(bool bSelect)
          type == OBJECT_MOBILEtt ||
          type == OBJECT_MOBILEwt ||
          type == OBJECT_MOBILEit ||
+         type == OBJECT_MOBILErp ||
+         type == OBJECT_MOBILEst ||
          type == OBJECT_MOBILEdr ||
          type == OBJECT_MOTHER   ||
          type == OBJECT_ANT      ||
@@ -929,8 +943,9 @@ bool CObjectInterface::CreateInterface(bool bSelect)
         }
     }
 
-    if ( type == OBJECT_HUMAN ||
-         type == OBJECT_TECH  )
+    if ( (type == OBJECT_HUMAN ||
+          type == OBJECT_TECH  ) &&
+         !m_main->GetPlusExplorer() )
     {
         pos.x = ox+sx*7.7f;
         pos.y = oy+sy*0.5f;
@@ -972,7 +987,7 @@ bool CObjectInterface::CreateInterface(bool bSelect)
         DefaultEnter(pw, EVENT_OBJECT_MTAKE);
     }
 
-    if ( type == OBJECT_HUMAN )  // builder?
+    if ( type == OBJECT_HUMAN && !m_main->GetPlusExplorer())  // builder?
     {
         pos.x  =   1.0f/640.0f;
         pos.y  =   4.0f/480.0f;
@@ -1050,8 +1065,8 @@ bool CObjectInterface::CreateInterface(bool bSelect)
 
         pos.x = ox+sx*5.4f;
         pos.y = oy+sy*0.1f;
-        pw->CreateButton(pos, ddim, 128+41, EVENT_OBJECT_BDESTROYER);
-        DeadInterface(pw, EVENT_OBJECT_BDESTROYER, m_main->CanBuild(OBJECT_DESTROYER, m_object->GetTeam()));
+        pw->CreateButton(pos, ddim, 128+47, EVENT_OBJECT_BSAFE);
+        DeadInterface(pw, EVENT_OBJECT_BSAFE, m_main->CanBuild(OBJECT_SAFE, m_object->GetTeam()));
 
         if ( m_main->IsBuildingEnabled(BUILD_GFLAT) )
         {
@@ -1102,16 +1117,39 @@ bool CObjectInterface::CreateInterface(bool bSelect)
         pw->CreateButton(pos, dim, 40, EVENT_OBJECT_SEARCH);
         DefaultEnter(pw, EVENT_OBJECT_SEARCH);
 
-        if ( m_main->IsBuildingEnabled(BUILD_GFLAT) )
-        {
-            pos.x = ox+sx*9.0f;
-            pos.y = oy+sy*0.5f;
-            pw->CreateButton(pos, dim, 111, EVENT_OBJECT_GFLAT);
-        }
-
-        pos.x = ox+sx*10.1f;
+        pos.x = ox+sx*9.0f;
         pos.y = oy+sy*0.5f;
         pw->CreateButton(pos, dim, 11, EVENT_OBJECT_DELSEARCH);
+
+        if ( m_main->IsBuildingEnabled(BUILD_FLAG) )
+        {
+            pos.x = ox+sx*10.1f;
+            pos.y = oy+sy*0.5f;
+            pw->CreateButton(pos, dim, 64+54, EVENT_OBJECT_FCREATE);
+
+            pos.x = ox+sx*11.1f;
+            pos.y = oy+sy*0.5f;
+            pw->CreateButton(pos, dim, 64+55, EVENT_OBJECT_FDELETE);
+
+            ddim.x = dim.x*0.4f;
+            ddim.y = dim.y*0.4f;
+            pos.x = ox+sx*10.1f;
+            pos.y = oy+sy*2.0f-ddim.y;
+            pc = pw->CreateColor(pos, ddim, -1, EVENT_OBJECT_FCOLORb);
+            pc->SetColor(Gfx::Color(0.28f, 0.56f, 1.0f, 0.0f));
+            pos.x += ddim.x;
+            pc = pw->CreateColor(pos, ddim, -1, EVENT_OBJECT_FCOLORr);
+            pc->SetColor(Gfx::Color(1.0f, 0.0f, 0.0f, 0.0f));
+            pos.x += ddim.x;
+            pc = pw->CreateColor(pos, ddim, -1, EVENT_OBJECT_FCOLORg);
+            pc->SetColor(Gfx::Color(0.0f, 0.8f, 0.0f, 0.0f));
+            pos.x += ddim.x;
+            pc = pw->CreateColor(pos, ddim, -1, EVENT_OBJECT_FCOLORy);
+            pc->SetColor(Gfx::Color(1.0f, 0.93f, 0.0f, 0.0f)); //0x00ffec00
+            pos.x += ddim.x;
+            pc = pw->CreateColor(pos, ddim, -1, EVENT_OBJECT_FCOLORv);
+            pc->SetColor(Gfx::Color(0.82f, 0.004f, 0.99f, 0.0f)); //0x00d101fe
+        }
     }
 
     if ( type == OBJECT_MOBILErt &&  // Terraformer?
@@ -1405,6 +1443,93 @@ bool CObjectInterface::CreateInterface(bool bSelect)
         pw->CreateGroup(pos, ddim, 16, EVENT_OBJECT_CORNERdr);
     }
 
+    if ( (type == OBJECT_MOBILEfb ||
+          type == OBJECT_MOBILEtb ||
+          type == OBJECT_MOBILEwb ||
+          type == OBJECT_MOBILEib) &&  // builder?
+         !m_object->GetTrainer() )
+    {
+        pos.x = ox+sx*7.7f;
+        pos.y = oy+sy*0.5f;
+        pb = pw->CreateButton(pos, dim, 192+4, EVENT_OBJECT_BUILD);
+        pb->SetImmediat(true);
+        DefaultEnter(pw, EVENT_OBJECT_BUILD);
+
+        pos.x = 0.0f;
+        pos.y = oy+sy*2.6f;
+        ddim.x = 214.5f/640.0f;
+        ddim.y = 66.0f/480.0f;
+        pw->CreateGroup(pos, ddim, 6, EVENT_WINDOW3);
+
+        ddim.x = dim.x*0.9f;
+        ddim.y = dim.y*0.9f;
+        pos.y = oy+sy*3.6f;
+
+        pos.x = ox+sx*0.0f;
+        pw->CreateButton(pos, ddim, 128+35, EVENT_OBJECT_BRESEARCH);
+        DeadInterface(pw, EVENT_OBJECT_BRESEARCH, m_main->CanBuild(OBJECT_RESEARCH, m_object->GetTeam()));
+
+        pos.x = ox+sx*0.9f;
+        pw->CreateButton(pos, ddim, 128+32, EVENT_OBJECT_BFACTORY);
+        DeadInterface(pw, EVENT_OBJECT_BFACTORY, m_main->CanBuild(OBJECT_FACTORY, m_object->GetTeam()));
+
+        pos.x = ox+sx*1.8f;
+        pw->CreateButton(pos, ddim, 128+34, EVENT_OBJECT_BCONVERT);
+        DeadInterface(pw, EVENT_OBJECT_BCONVERT, m_main->CanBuild(OBJECT_CONVERT, m_object->GetTeam()));
+
+        pos.x = ox+sx*2.7f;
+        pw->CreateButton(pos, ddim, 128+36, EVENT_OBJECT_BSTATION);
+        DeadInterface(pw, EVENT_OBJECT_BSTATION, m_main->CanBuild(OBJECT_STATION, m_object->GetTeam()));
+
+        pos.x = ox+sx*3.6f;
+        pw->CreateButton(pos, ddim, 128+40, EVENT_OBJECT_BRADAR);
+        DeadInterface(pw, EVENT_OBJECT_BRADAR, m_main->CanBuild(OBJECT_RADAR, m_object->GetTeam()));
+
+        pos.x = ox+sx*4.5f;
+        pw->CreateButton(pos, ddim, 128+41, EVENT_OBJECT_BREPAIR);
+        DeadInterface(pw, EVENT_OBJECT_BREPAIR, m_main->CanBuild(OBJECT_REPAIR, m_object->GetTeam()));
+
+        pos.x = ox+sx*5.4f;
+        pw->CreateButton(pos, ddim, 128+44, EVENT_OBJECT_BINFO);
+        DeadInterface(pw, EVENT_OBJECT_BINFO, m_main->CanBuild(OBJECT_INFO, m_object->GetTeam()));
+
+        pos.y = oy+sy*2.7f;
+
+        pos.x = ox+sx*0.0f;
+        pw->CreateButton(pos, ddim, 128+37, EVENT_OBJECT_BTOWER);
+        DeadInterface(pw, EVENT_OBJECT_BTOWER, m_main->CanBuild(OBJECT_TOWER, m_object->GetTeam()));
+
+        pos.x = ox+sx*0.9f;
+        pw->CreateButton(pos, ddim, 128+39, EVENT_OBJECT_BENERGY);
+        DeadInterface(pw, EVENT_OBJECT_BENERGY, m_main->CanBuild(OBJECT_ENERGY, m_object->GetTeam()));
+
+        pos.x = ox+sx*1.8f;
+        pw->CreateButton(pos, ddim, 128+33, EVENT_OBJECT_BDERRICK);
+        DeadInterface(pw, EVENT_OBJECT_BDERRICK, m_main->CanBuild(OBJECT_DERRICK, m_object->GetTeam()));
+
+        pos.x = ox+sx*2.7f;
+        pw->CreateButton(pos, ddim, 128+42, EVENT_OBJECT_BNUCLEAR);
+        DeadInterface(pw, EVENT_OBJECT_BNUCLEAR, m_main->CanBuild(OBJECT_NUCLEAR, m_object->GetTeam()));
+
+        pos.x = ox+sx*3.6f;
+        pw->CreateButton(pos, ddim, 128+38, EVENT_OBJECT_BLABO);
+        DeadInterface(pw, EVENT_OBJECT_BLABO, m_main->CanBuild(OBJECT_LABO, m_object->GetTeam()));
+
+        pos.x = ox+sx*4.5f;
+        pw->CreateButton(pos, ddim, 128+46, EVENT_OBJECT_BPARA);
+        DeadInterface(pw, EVENT_OBJECT_BPARA, m_main->CanBuild(OBJECT_PARA, m_object->GetTeam()));
+
+        pos.x = ox+sx*5.4f;
+        pw->CreateButton(pos, ddim, 128+47, EVENT_OBJECT_BSAFE);
+        DeadInterface(pw, EVENT_OBJECT_BSAFE, m_main->CanBuild(OBJECT_SAFE, m_object->GetTeam()));
+
+        if ( m_main->IsBuildingEnabled(BUILD_GFLAT) )
+        {
+            pos.x = ox+sx*9.0f;
+            pos.y = oy+sy*0.5f;
+            pw->CreateButton(pos, dim, 64+47, EVENT_OBJECT_GFLAT);
+        }
+    }
     UpdateInterface();
     m_lastUpdateTime = 0.0f;
     UpdateInterface(0.0f);
@@ -1500,7 +1625,7 @@ void CObjectInterface::UpdateInterface(float rTime)
                 m_lastAlarmTime = 0.0f;
             }
         }
-        
+
         pg->SetLevel(shield);
         pg->SetIcon(icon);
     }
@@ -1664,6 +1789,7 @@ void CObjectInterface::UpdateInterface()
     EnableInterface(pw, EVENT_OBJECT_TERRAFORM,   bEnable);
     EnableInterface(pw, EVENT_OBJECT_RECOVER,     bEnable);
     EnableInterface(pw, EVENT_OBJECT_FIRE,        bEnable);
+    EnableInterface(pw, EVENT_OBJECT_BUILD,       bEnable);
     EnableInterface(pw, EVENT_OBJECT_SPIDEREXPLO, bEnable);
     EnableInterface(pw, EVENT_OBJECT_RESET,       bEnable);
     EnableInterface(pw, EVENT_OBJECT_PEN0,        bEnable);
@@ -1678,7 +1804,11 @@ void CObjectInterface::UpdateInterface()
     EnableInterface(pw, EVENT_OBJECT_REC,         bEnable);
     EnableInterface(pw, EVENT_OBJECT_STOP,        bEnable);
 
-    if ( type == OBJECT_HUMAN )  // builder?
+    if ( type == OBJECT_HUMAN    ||  // builder?
+         type == OBJECT_MOBILEfb ||
+         type == OBJECT_MOBILEtb ||
+         type == OBJECT_MOBILEwb ||
+         type == OBJECT_MOBILEib )
     {
         EnableInterface(pw, EVENT_OBJECT_BFACTORY,  bEnable);
         EnableInterface(pw, EVENT_OBJECT_BDERRICK,  bEnable);
@@ -1693,11 +1823,15 @@ void CObjectInterface::UpdateInterface()
         EnableInterface(pw, EVENT_OBJECT_BNUCLEAR,  bEnable);
         EnableInterface(pw, EVENT_OBJECT_BPARA,     bEnable);
         EnableInterface(pw, EVENT_OBJECT_BINFO,     bEnable);
-        EnableInterface(pw, EVENT_OBJECT_BDESTROYER,bEnable);
+        EnableInterface(pw, EVENT_OBJECT_BSAFE,     bEnable);
     }
 
-    if ( type == OBJECT_HUMAN ||  // builder?
-         type == OBJECT_TECH  )
+    if ( type == OBJECT_HUMAN    ||  // can create flags?
+         type == OBJECT_TECH     ||
+         type == OBJECT_MOBILEfs ||
+         type == OBJECT_MOBILEts ||
+         type == OBJECT_MOBILEws ||
+         type == OBJECT_MOBILEis )
     {
         CheckInterface(pw, EVENT_OBJECT_FCOLORb, m_flagColor==0);
         CheckInterface(pw, EVENT_OBJECT_FCOLORr, m_flagColor==1);
@@ -1726,10 +1860,52 @@ void CObjectInterface::UpdateInterface()
         ps = static_cast< CSlider* >(pw->SearchControl(EVENT_OBJECT_DIMSHIELD));
         if ( ps != nullptr )
         {
-            ps->SetVisibleValue((RADIUS_SHIELD_MIN/g_unit)+dynamic_cast<CShielder*>(m_object)->GetShieldRadius()*((RADIUS_SHIELD_MAX-RADIUS_SHIELD_MIN)/g_unit));
+            ps->SetVisibleValue((RADIUS_SHIELD_MIN/g_unit)+dynamic_cast<CShielder&>(*m_object).GetShieldRadius()*((RADIUS_SHIELD_MAX-RADIUS_SHIELD_MIN)/g_unit));
         }
     }
 
+    if ( (type == OBJECT_MOBILEfb ||
+          type == OBJECT_MOBILEtb ||
+          type == OBJECT_MOBILEwb ||
+          type == OBJECT_MOBILEib) &&  // builder?
+         !m_object->GetTrainer() )
+    {
+        if(!bEnable) m_buildInterface = false;
+        CheckInterface(pw, EVENT_OBJECT_BUILD, m_buildInterface);
+
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_WINDOW3));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BFACTORY));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BDERRICK));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BCONVERT));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BSTATION));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BREPAIR));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BTOWER));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BRESEARCH));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BRADAR));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BENERGY));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BLABO));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BNUCLEAR));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BPARA));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BSAFE));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+        pb = static_cast< CButton* >(pw->SearchControl(EVENT_OBJECT_BINFO));
+        pb->SetState(STATE_VISIBLE, m_buildInterface);
+    }
+
     bFly = bEnable;
     if ( bFly && (type == OBJECT_HUMAN || type == OBJECT_TECH) )
     {
@@ -1755,6 +1931,10 @@ void CObjectInterface::UpdateInterface()
          type == OBJECT_MOBILEta ||
          type == OBJECT_MOBILEwa ||
          type == OBJECT_MOBILEia ||
+         type == OBJECT_MOBILEfb ||
+         type == OBJECT_MOBILEtb ||
+         type == OBJECT_MOBILEwb ||
+         type == OBJECT_MOBILEib ||
          type == OBJECT_MOBILEfc ||
          type == OBJECT_MOBILEtc ||
          type == OBJECT_MOBILEwc ||
@@ -1777,6 +1957,8 @@ void CObjectInterface::UpdateInterface()
          type == OBJECT_MOBILEtt ||
          type == OBJECT_MOBILEwt ||
          type == OBJECT_MOBILEit ||
+         type == OBJECT_MOBILErp ||
+         type == OBJECT_MOBILEst ||
          type == OBJECT_MOBILEdr ||
          type == OBJECT_MOTHER   ||
          type == OBJECT_ANT      ||
