@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -39,7 +39,7 @@ protected:
     void IOUpdateList(bool isWrite);
     void IODeleteScene();
     void IOWriteScene();
-    void IOReadScene();
+    bool IOReadScene();
 
 protected:
     CScreenLevelList* m_screenLevelList;
