@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -91,14 +91,22 @@ void CScreenMainMenu::CreateInterface()
     pg = pw->CreateGroup(pos, ddim, 26, EVENT_LABEL1);  // quit
     pg->SetState(STATE_SHADOW);
 
-    ddim.x = 0.18f;
+    ddim.x = 0.12f;
     ddim.y = dim.y*0.8f;
     pos.x = 0.41f;
-
     pos.y = oy+sy*10.5f;
+
     pb = pw->CreateButton(pos, ddim, -1, EVENT_INTERFACE_MISSION);
     pb->SetState(STATE_SHADOW);
 
+    ddim.x = 0.06f;
+    pos.x = 0.53f;
+    pb = pw->CreateButton(pos, ddim, -1, EVENT_INTERFACE_PLUS);
+    pb->SetState(STATE_SHADOW);
+
+    ddim.x = 0.18f;
+    pos.x = 0.41f;
+
     pos.y = oy+sy*9.6f;
     pb = pw->CreateButton(pos, ddim, -1, EVENT_INTERFACE_FREE);
     pb->SetState(STATE_SHADOW);
@@ -159,7 +167,7 @@ void CScreenMainMenu::CreateInterface()
     pg->SetState(STATE_SHADOW);
     pos.y -=  5.0f/480.0f;
     pl = pw->CreateLabel(pos, ddim, 0, EVENT_LABEL1, "TerranovaTeam");
-    pl->SetFontType(Gfx::FONT_COURIER);
+    pl->SetFontType(Gfx::FONT_STUDIO);
     pl->SetFontSize(Gfx::FONT_SIZE_SMALL);
 
     // SatCom button
@@ -170,6 +178,13 @@ void CScreenMainMenu::CreateInterface()
     pb = pw->CreateButton(pos, ddim, 128+60, EVENT_INTERFACE_SATCOM);
     pb->SetState(STATE_SHADOW);
 
+    // Mods button
+    pos.x  = 447.0f/640.0f;
+    pos.y  = 313.0f/480.0f;
+    ddim.x = 0.09f;
+    pb = pw->CreateButton(pos, ddim, -1, EVENT_INTERFACE_MODS);
+    pb->SetState(STATE_SHADOW);
+
     SetBackground("textures/interface/interface.png");
     CreateVersionDisplay();
 }
@@ -218,6 +233,11 @@ bool CScreenMainMenu::EventProcess(const Event &event)
             m_main->ChangePhase(PHASE_LEVEL_LIST);
             break;
 
+        case EVENT_INTERFACE_PLUS:
+            m_main->SetLevel(LevelCategory::GamePlus, 0, 0);
+            m_main->ChangePhase(PHASE_LEVEL_LIST);
+            break;
+
         case EVENT_INTERFACE_USER:
             m_main->SetLevel(LevelCategory::CustomLevels, 0, 0);
             m_main->ChangePhase(PHASE_LEVEL_LIST);
@@ -235,6 +255,9 @@ bool CScreenMainMenu::EventProcess(const Event &event)
             m_main->ChangePhase(PHASE_SATCOM);
             break;
 
+        case EVENT_INTERFACE_MODS:
+            m_main->ChangePhase(PHASE_MOD_LIST);
+
         default:
             return true;
     }
