@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -27,12 +27,12 @@ using namespace CBot;
 class CBotTokenUT : public testing::Test
 {
 public:
-    void SetUp()
+    CBotTokenUT()
     {
         CBotProgram::Init();
     }
 
-    void TearDown()
+    ~CBotTokenUT()
     {
         CBotProgram::Free();
     }
